!$Header: /home/ssipsl/CVSREP/ORCHIDEE/src_stomate/stomate.f90,v 1.33.4.1 2009/01/26 07:46:14 ssipsl Exp $
!IPSL (2006)
! This software is governed by the CeCILL licence see ORCHIDEE/ORCHIDEE_CeCILL.LIC
!
MODULE stomate
!---------------------------------------------------------------------
! Daily update of leaf area index etc.
!---------------------------------------------------------------------
  USE netcdf
!-
  USE ioipsl
  USE defprec
  USE constantes
  USE constantes_veg
  USE constantes_co2
  USE stomate_constants
  USE stomate_io
  USE stomate_data
  USE stomate_natagritot
  USE stomate_season
  USE stomate_lpj
  USE stomate_assimtemp
  USE stomate_litter
  USE stomate_vmax
  USE stomate_soilcarbon
  USE stomate_resp
  USE parallel
!  USE Write_field_p
!-
  IMPLICIT NONE
  PRIVATE
  PUBLIC stomate_soil,stomate_main,stomate_clear, &
 &       pondere_nat,pondere_nat_vegfrac,pondere_vegfrac
  !
  INTEGER,PARAMETER :: r_typ =nf90_real4

  !
  ! Do update of yearly variables ?
  ! This variable must be .TRUE. once a year
  LOGICAL, SAVE                                   :: EndOfYear = .FALSE.
  PUBLIC EndOfYear
!-
! variables used inside stomate module : declaration and initialisation
!-
! total natural space (fraction of total space)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: veget_max
! total natural space (fraction of total space)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)      :: space_nat
! carbon pool: active, slow, or passive, natural and agricultural
! (gC/(m**2 of nat/agri ground))
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:)  :: carbon
! density of individuals (1/(m**2 of nat/agri ground))
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: ind
! daily moisture availability
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: humrel_daily
! daily litter humidity
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)      :: litterhum_daily
! daily 2 meter temperatures (K)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)      :: t2m_daily
! daily minimum 2 meter temperatures (K)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)      :: t2m_min_daily
! daily surface temperatures (K)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)      :: tsurf_daily
! daily soil temperatures (K)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: tsoil_daily
! daily soil humidity
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: soilhum_daily
! daily precipitations (mm/day) (for phenology)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)      :: precip_daily
! daily gross primary productivity (gC/(m**2 of nat/agri ground)/day)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: gpp_daily
! daily net primary productivity (gC/(m**2 of nat/agri ground)/day)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: npp_daily
! Turnover rates (gC/(m**2 of nat/agri ground)/day)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:)  :: turnover_daily
  ! Turnover rates (gC/(m**2 of ground)*dtradia)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:)   :: turnover_littercalc
! Probability of fire
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: fireindex
! Longer term total litter above the ground
! (gC/m**2 of nat/agri ground)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: firelitter
! "monthly" moisture availability
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: humrel_month
! "weekly" moisture availability
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: humrel_week
! "long term" 2 meter temperatures (K)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)      :: t2m_longterm
! "long term" 2 meter reference temperatures (K)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)      :: tlong_ref
! "monthly" 2 meter temperatures (K)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)      :: t2m_month
! "weekly" 2 meter temperatures (K)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)      :: t2m_week
! "monthly" soil temperatures (K)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: tsoil_month
! "monthly" soil humidity
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: soilhum_month
! growing degree days, threshold -5 deg C (for phenology)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: gdd_m5_dormance
! growing degree days, since midwinter (for phenology)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: gdd_midwinter
! number of chilling days since leaves were lost (for phenology)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: ncd_dormance
! number of growing days, threshold -5 deg C (for phenology)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: ngd_minus5
! last year's maximum moisture availability
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: maxhumrel_lastyear
! this year's maximum moisture availability
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: maxhumrel_thisyear
! last year's minimum moisture availability
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: minhumrel_lastyear
! this year's minimum moisture availability
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: minhumrel_thisyear
! last year's maximum "weekly" GPP (gC/m**2 covered/day)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: maxgppweek_lastyear
! this year's maximum "weekly" GPP (gC/m**2 covered/day)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: maxgppweek_thisyear
! last year's annual GDD0
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)      :: gdd0_lastyear
! this year's annual GDD0
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)      :: gdd0_thisyear
! last year's annual precipitation (mm/year)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)      :: precip_lastyear
! this year's annual precipitation (mm/year)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)      :: precip_thisyear
! PFT exists (equivalent to veget > 0 for natural PFTs)
  LOGICAL,ALLOCATABLE,SAVE,DIMENSION(:,:)        :: PFTpresent
! "long term" net primary productivity
! (gC/(m**2 of nat/agri ground)/year)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: npp_longterm
! last year's maximum leaf mass, for each PFT
! (gC/(m**2 of nat/agri ground))
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: lm_lastyearmax
! this year's maximum leaf mass, for each PFT
! (gC/(m**2 of nat/agri ground))
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: lm_thisyearmax
! last year's maximum fpc for each natural PFT, on *natural* ground
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: maxfpc_lastyear
! this year's maximum fpc for each PFT, on *total* ground
! (see stomate_season)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: maxfpc_thisyear
! "long term" turnover rate (gC/(m**2 of nat/agri ground)/year)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:)  :: turnover_longterm
! "weekly" GPP (gC/day/(m**2 covered)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: gpp_week
! biomass (gC/(m**2 of nat/agri ground))
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:)  :: biomass
! is the plant senescent?
! (only for deciduous trees - carbohydrate reserve)
  LOGICAL,ALLOCATABLE,SAVE,DIMENSION(:,:)        :: senescence
! how many days ago was the beginning of the growing season
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: when_growthinit
! age (years). For trees, mean stand age.
! For grasses, ears since introduction of PFT
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: age
! Winter too cold? between 0 and 1
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: adapted
! Winter sufficiently cold? between 0 and 1
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: regenerate
! fraction of litter above the ground belonging to different PFTs,
! separated for natural and agricultural PFTs
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:)  :: litterpart
! metabolic and structural litter,
! natural and agricultural, above and below ground
!   (gC/(m**2 of nat/agri ground))
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:,:):: litter
! dead leaves on ground, per PFT, metabolic and structural,
! in gC/(m**2 of nat/agri ground)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:)  :: dead_leaves
! black carbon on the ground (gC/(m**2 of total ground))
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)      :: black_carbon
! ratio Lignine/Carbon in structural litter,
! above and below ground, natural and agricultural
!   (gC/(m**2 of nat/agri ground))
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:)  :: lignin_struc
! carbon emitted into the atmosphere by fire (living and dead biomass)
!   (in gC/m**2 of average ground/day)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)      :: co2_fire
! co2 taken up (gC/(m**2 of total ground)/day)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)      :: co2_to_bm_dgvm
! heterotrophic respiration (gC/day/m**2 of total ground)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: resp_hetero
! maintenance respiration (gC/day/(m**2 of total ground))
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: resp_hetero_radia
! maintenance respiration (gC/day/(m**2 of total ground))
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: resp_maint
! growth respiration (gC/day/(m**2 of total ground))
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: resp_growth
! vegetation fractions (on natural/agri ground)
! after last light competition
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: veget_lastlight
! is the PFT everywhere in the grid box or very localized
! (after its intoduction)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: everywhere
! in order for this PFT to be introduced,
! does it have to be present in an adjacent grid box?
  LOGICAL,ALLOCATABLE,SAVE,DIMENSION(:,:)       :: need_adjacent
! leaf age (d)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:)  :: leaf_age
! fraction of leaves in leaf age class
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:)  :: leaf_frac
! How much time ago was the PFT eliminated for the last time (y)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: RIP_time
! duration of dormance (d)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: time_lowgpp
! time elapsed since strongest moisture availability (d)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: time_hum_min
! minimum moisture during dormance
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: hum_min_dormance
! factor that allows to transform "veget_max_x" (Sechiba)
! into "veget_max" (Stomate).
! veget_max_x takes into account the ice fraction,
! while veget_max is defined on ice_free
! land surface only. Moreover,
! PFTs between Sechiba and Stomate are not necessarily identical.
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)  :: fvm
! factor that allows to transform
! "veget_x" (Sechiba) into "veget" (Stomate) [see "fvm"].
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)  :: fv
! time constant of probability of a leaf to be eaten
! by a herbivore (days)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)  :: herbivores
! npp total written for forcesoil...
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)  :: npp_equil
! npp total ...
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)  :: npp_tot
! moisture control of heterotrophic respiration
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:) :: control_moist
! temperature control of heterotrophic respiration, above and below
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:) :: control_temp
! quantity of carbon going into carbon pools from litter decomposition
! (gC/(m**2 of nat/agri ground)/day)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:,:) :: soilcarbon_input
! times at which soil forcing file is written
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)      :: times
! how many states were calculated for a given soil forcing time step
! turnover time of leaves
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)    :: turnover_time
! daily total CO2 flux (gC/m**2/day)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)      :: co2_flux_daily 
! monthly total CO2 flux (gC/m**2)
 REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)       :: co2_flux_monthly
! conversion of biomass to litter (gC/(m**2 of nat/agri ground))/day
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:)  :: bm_to_litter
  ! conversion of biomass to litter (gC/(m**2 of ground))*dtradia
  REAL(r_std), ALLOCATABLE,SAVE,DIMENSION(:,:,:)  :: bm_to_littercalc

  INTEGER(i_std),ALLOCATABLE,SAVE,DIMENSION(:)  :: nforce

! forcing data in memory
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:)     :: clay_fm
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:,:)   :: humrel_daily_x_fm
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:)     :: litterhum_daily_fm
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:)     :: t2m_daily_fm
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:)     :: t2m_min_daily_fm
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:)     :: tsurf_daily_fm
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:,:)   :: tsoil_daily_fm
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:,:)   :: soilhum_daily_fm
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:)     :: precip_fm
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:,:)   :: gpp_daily_x_fm
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:,:,:) :: resp_maint_part_x_fm
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:,:)   :: veget_x_fm
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:,:)   :: veget_max_x_fm
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:,:)   :: lai_x_fm
  
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:)     :: clay_fm_g
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:,:)   :: humrel_daily_x_fm_g
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:)     :: litterhum_daily_fm_g
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:)     :: t2m_daily_fm_g
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:)     :: t2m_min_daily_fm_g
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:)     :: tsurf_daily_fm_g
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:,:)   :: tsoil_daily_fm_g
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:,:)   :: soilhum_daily_fm_g
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:)     :: precip_fm_g
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:,:)   :: gpp_daily_x_fm_g
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:,:,:) :: resp_maint_part_x_fm_g
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:,:)   :: veget_x_fm_g
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:,:)   :: veget_max_x_fm_g
  REAL(r_std),ALLOCATABLE,DIMENSION(:,:,:)   :: lai_x_fm_g
  
  INTEGER(i_std),ALLOCATABLE,SAVE,DIMENSION(:) :: isf
  LOGICAL,ALLOCATABLE,SAVE,DIMENSION(:)      :: nf_written
  INTEGER(i_std),ALLOCATABLE,SAVE,DIMENSION(:) :: nf_cumul
! first call
  LOGICAL,SAVE :: l_first_stomate = .TRUE.
! flag for cumul of forcing if teststomate
  LOGICAL,SAVE :: cumul_forcing=.FALSE.
!
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:) :: resp_maint_part_radia
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:,:) :: resp_maint_part
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)   :: resp_maint_radia

! deforestation variables
! products remaining in the 10/100 year-turnover pool after the annual release for each compartment
! (10 or 100 + 1 : input from year of deforestation)
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)                         :: prod10
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)                         :: prod100
! annual release from the 10/100 year-turnover pool compartments
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)                          :: flux10
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:,:)                          :: flux100
! new total natural space (fraction of total space)
  REAL(r_std),ALLOCATABLE,SAVE, DIMENSION(:)                           :: space_nat_new
! new "maximal" coverage fraction of a PFT (LAI -> infinity) on nat/agri ground
  REAL(r_std),ALLOCATABLE,SAVE, DIMENSION(:,:)                         :: veget_max_new 
! release during first year following deforestation
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)                            :: convflux
! total annual release from the 10/100 year-turnover pool
  REAL(r_std),ALLOCATABLE,SAVE,DIMENSION(:)                            :: cflux_prod10, cflux_prod100
! deforestation (land cover change) flag
  LOGICAL,SAVE                            :: defor

CONTAINS
!
!=
!
  SUBROUTINE stomate_main &
       & (kjit, kjpij, kjpindex, dtradia, dt_slow, &
       &  ldrestart_read, ldrestart_write, ldforcing_write, ldcarbon_write, &
       &  index, lalo, neighbours, resolution, contfrac, fraction_nobio, clay, &
       &  t2m, t2m_min, temp_sol, stempdiag, &
       &  humrel_x, shumdiag, litterhumdiag, precip_rain, precip_snow, &
       &  gpp_x, deadleaf_cover, assim_param_x, &
       &  lai_x, height_x, veget_x, veget_max_x, qsintmax, &
       &  hist_id, hist2_id, rest_id_stom, hist_id_stom, &
       &  co2_flux,resp_maint_x,resp_hetero_x,resp_growth_x)
    !---------------------------------------------------------------------
    !
    ! 0 interface description
    !
    ! 0.1 input
    !
    ! 0.1.1 input scalar
    !
    ! Time step number
    INTEGER(i_std),INTENT(in)                   :: kjit
    ! Domain size
    INTEGER(i_std),INTENT(in)                   :: kjpindex
    ! Total size of the un-compressed grid
    INTEGER(i_std),INTENT(in)                   :: kjpij
    ! Time step of SECHIBA
    REAL(r_std),INTENT(in)                    :: dtradia
    ! Time step of STOMATE
    REAL(r_std),INTENT(in)                    :: dt_slow
    ! Logical for _restart_ file to read
    LOGICAL,INTENT(in)                        :: ldrestart_read
    ! Logical for _restart_ file to write
    LOGICAL,INTENT(in)                        :: ldrestart_write
    ! Logical for _forcing_ file to write
    LOGICAL,INTENT(in)                        :: ldforcing_write
    ! Logical for _carbon_forcing_ file to write
    LOGICAL,INTENT(in)                        :: ldcarbon_write
    ! SECHIBA's _history_ file identifier
    INTEGER(i_std),INTENT(in)                   :: hist_id
    ! SECHIBA's _history_ file 2 identifier
    INTEGER(i_std),INTENT(in)                   :: hist2_id
    ! STOMATE's _Restart_ file file identifier
    INTEGER(i_std),INTENT(in)                   :: rest_id_stom
    ! STOMATE's _history_ file file identifier
    INTEGER(i_std),INTENT(in)                   :: hist_id_stom
    !
    ! 0.1.2 input fields
    !
    ! Indeces of the points on the map
    INTEGER(i_std),DIMENSION(kjpindex),INTENT(in)  :: index
    ! Geogr. coordinates (latitude,longitude) (degrees)
    REAL(r_std),DIMENSION(kjpindex,2),INTENT(in)     :: lalo
    ! neighoring grid points if land
    INTEGER(i_std),DIMENSION(kjpindex,8),INTENT(in) :: neighbours
    ! size in x an y of the grid (m)
    REAL(r_std),DIMENSION(kjpindex,2),INTENT(in)     :: resolution
    ! fraction of continent in the grid
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)    :: contfrac
    ! fraction of grid cell covered by lakes, land ice, cities, ...
    REAL(r_std),DIMENSION(kjpindex),INTENT(in)       :: fraction_nobio
    ! clay fraction
    REAL(r_std),DIMENSION(kjpindex),INTENT(in)       :: clay
    ! Relative humidity ("moisture availability")
    REAL(r_std),DIMENSION(kjpindex,nvm),INTENT(in)   :: humrel_x
    ! 2 m air temperature (K)
    REAL(r_std),DIMENSION(kjpindex),INTENT(in)       :: t2m
    ! min. 2 m air temp. during forcing time step (K)
    REAL(r_std),DIMENSION(kjpindex),INTENT(in)       :: t2m_min
    ! Surface temperature
    REAL(r_std),DIMENSION(kjpindex),INTENT(in)       :: temp_sol
    ! Soil temperature
    REAL(r_std),DIMENSION(kjpindex,nbdl),INTENT(in)  :: stempdiag
    ! Relative soil moisture
    REAL(r_std),DIMENSION(kjpindex,nbdl),INTENT(in)  :: shumdiag
    ! Litter humidity
    REAL(r_std),DIMENSION(kjpindex),INTENT(in)       :: litterhumdiag
    ! Rain precipitation
    REAL(r_std),DIMENSION(kjpindex),INTENT(in)       :: precip_rain
    ! Snow precipitation
    REAL(r_std),DIMENSION(kjpindex),INTENT(in)       :: precip_snow
    ! GPP (gC/(m**2 of total ground)/time step)
    REAL(r_std),DIMENSION(kjpindex,nvm),INTENT(in)   :: gpp_x
    !
    ! 0.2 output
    !
    ! 0.2.1 output scalar
    !
    ! 0.2.2 output fields
    !
    ! CO2 flux in gC/m**2 of average ground/dt
    REAL(r_std),DIMENSION(kjpindex),INTENT(out)      :: co2_flux
    ! autotrophic respiration in gC/m**2 of surface/dt
    REAL(r_std),DIMENSION(kjpindex,nvm),INTENT(out)  :: resp_maint_x
    REAL(r_std),DIMENSION(kjpindex,nvm),INTENT(out)  :: resp_growth_x
    REAL(r_std),DIMENSION(kjpindex),INTENT(out)      :: resp_hetero_x
    !
    ! 0.3 modified
    !
    ! 0.3.1 modified scalar
    ! 0.3.2 modified fields
    !
    ! Surface foliere
    REAL(r_std),DIMENSION(kjpindex,nvm),INTENT(inout) :: lai_x
    ! Fraction of vegetation type from hydrological module.
    ! Takes into account ice etc.
    REAL(r_std),DIMENSION(kjpindex,nvm),INTENT(inout) :: veget_x
    ! Maximum fraction of vegetation type from hydrological module.
    ! Takes into account ice etc.
    REAL(r_std),DIMENSION(kjpindex,nvm),INTENT(inout) :: veget_max_x
    ! height (m)
    REAL(r_std),DIMENSION(kjpindex,nvm),INTENT(inout) :: height_x
    ! min+max+opt temps & vmax for photosynthesis
    REAL(r_std),DIMENSION(kjpindex,nvm,npco2),INTENT(inout):: assim_param_x
    ! fraction of soil covered by dead leaves
    REAL(r_std),DIMENSION(kjpindex),INTENT(inout)    :: deadleaf_cover
    ! Maximum water on vegetation for interception
    REAL(r_std),DIMENSION(kjpindex,nvm),INTENT(inout) :: qsintmax
    !
    ! 0.4 local declaration
    !
    ! 0.4.1 variables defined on nvm in SECHIBA, npft in STOMATE/LPJ
    !
    ! moisture availability
    REAL(r_std),DIMENSION(kjpindex,npft)             :: humrel
    ! gross primary productivity (gC/(m**2 of total ground)/day)
    REAL(r_std),DIMENSION(kjpindex,npft)             :: gpp
    ! fractional coverage: actually covered space,
    ! taking into account LAI (= grid scale fpc).
    ! Fraction of nat/agri ground.
    REAL(r_std),DIMENSION(kjpindex,npft)             :: veget
    !
    ! 0.4.2 other
    !
    ! time step of STOMATE in days
    REAL(r_std),SAVE                              :: dt_days
    ! to check
    REAL(r_std),SAVE                              :: day_counter
    ! date (d)
    INTEGER(i_std),SAVE                          :: date
    ! soil level used for LAI
    INTEGER(i_std),SAVE                          :: lcanop
    ! STOMATE time step read in restart file
    REAL(r_std)                                   :: dt_days_read
    ! Maximum STOMATE time step (days)
    REAL(r_std),PARAMETER                         :: max_dt_days = 5.
    ! is it time for Stomate or update of LAI etc. ?
    LOGICAL                                      :: do_slow
    ! Writing frequency for history file (d)
    REAL(r_std)                                   :: hist_days
    ! precipitation (mm/day)
    REAL(r_std),DIMENSION(kjpindex)               :: precip
    ! Maximum rate of carboxylation
    REAL(r_std),DIMENSION(kjpindex,npft)          :: vcmax
    ! Maximum rate of RUbp regeneration
    REAL(r_std),DIMENSION(kjpindex,npft)          :: vjmax
    ! Min temperature for photosynthesis (deg C)
    REAL(r_std),DIMENSION(kjpindex,npft)          :: t_photo_min
    ! Opt temperature for photosynthesis (deg C)
    REAL(r_std),DIMENSION(kjpindex,npft)          :: t_photo_opt
    ! Max temperature for photosynthesis (deg C)
    REAL(r_std),DIMENSION(kjpindex,npft)          :: t_photo_max
    ! leaf area index
    REAL(r_std),DIMENSION(kjpindex,npft)          :: lai
    ! total autotrophic respiration (gC/day/(m**2 of total ground))
    REAL(r_std),DIMENSION(kjpindex)               :: resp_auto_tot
    ! total photosynthesis (gC/day/(m**2 of total ground))
    REAL(r_std),DIMENSION(kjpindex)               :: gpp_tot
    ! -- LOOP
    REAL(r_std)                                   :: net_co2_flux_monthly, net_co2_flux_monthly_sum
    INTEGER                                      :: ios
    ! for forcing file: "daily" moisture availability
    REAL(r_std),DIMENSION(kjpindex,nvm)           :: humrel_daily_x
    ! for forcing file: "daily" gpp
    REAL(r_std),DIMENSION(kjpindex,nvm)           :: gpp_daily_x
    ! for forcing file: "daily" auto resp
    REAL(r_std),DIMENSION(kjpindex,nvm,nparts)    :: resp_maint_part_x
    ! total "vegetation" cover
    REAL(r_std),DIMENSION(kjpindex)               :: cvegtot
    ! height of vegetation (m)
    REAL(r_std),DIMENSION(kjpindex,npft)          :: height
    INTEGER(i_std)                               :: ji, jv, i, j
    REAL(r_std)                                   :: trans_veg
    REAL(r_std)                                   :: tmp_day(1)
    !
    
    INTEGER(i_std)                               :: ier
    ! moisture control of heterotrophic respiration
    REAL(r_std),DIMENSION(kjpindex,nlevs)         :: control_moist_inst
    ! temperature control of heterotrophic respiration, above and below
    REAL(r_std),DIMENSION(kjpindex,nlevs)         :: control_temp_inst
    ! quantity of carbon going into carbon pools from litter decomposition
    ! (gC/(m**2 of nat/agri ground)/day)
    REAL(r_std),DIMENSION(kjpindex,ncarb,nvegtypes):: soilcarbon_input_inst
    ! time step of soil forcing file (in days)
    REAL(r_std),SAVE            :: dt_forcesoil
    INTEGER(i_std),SAVE        :: nparan            ! Number of time steps per year for carbon spinup
    INTEGER(i_std),SAVE        :: nbyear
    INTEGER(i_std),PARAMETER   :: nparanmax=36      ! Number max of time steps per year for carbon spinup
    REAL(r_std)                 :: sf_time
    INTEGER(i_std),SAVE        :: iatt=1
    INTEGER(i_std)             :: max_totsize, totsize_1step,totsize_tmp
    REAL(r_std)                 :: xn
    INTEGER(i_std),SAVE        :: nsfm, nsft
    INTEGER(i_std),SAVE        :: iisf
    !-
    CHARACTER(LEN=100), SAVE    :: forcing_name,Cforcing_name
    INTEGER(i_std),SAVE         :: Cforcing_id
    INTEGER(i_std),PARAMETER    :: ndm = 10
    INTEGER(i_std)              :: vid
    INTEGER(i_std)              :: nneigh,direct
    INTEGER(i_std),DIMENSION(ndm) :: d_id


    ! root temperature (convolution of root and soil temperature profiles)
    REAL(r_std),DIMENSION(kjpindex,npft)            :: t_root
    REAL(r_std),DIMENSION(kjpindex,npft,nparts)     :: coeff_maint
    ! temperature which is pertinent for maintenance respiration (K)
    REAL(r_std),DIMENSION(kjpindex,nparts)          :: t_maint_radia
    ! integration constant for root profile
    REAL(r_std),DIMENSION(kjpindex)                 :: rpc
    ! long term annual mean temperature, C
    REAL(r_std),DIMENSION(kjpindex)                 :: tl
    ! slope of maintenance respiration coefficient (1/K)
    REAL(r_std),DIMENSION(kjpindex)                 :: slope
    ! soil levels (m)
    REAL(r_std),DIMENSION(0:nbdl)                   :: z_soil
    ! root depth. This will, one day, be a prognostic variable.
    ! It will be calculated by
    ! STOMATE (save in restart file & give to hydrology module!),
    ! probably somewhere
    ! in the allocation routine. For the moment, it is prescribed.
    REAL(r_std),DIMENSION(kjpindex,npft)            :: rprof
    INTEGER(i_std)                                 :: l,k
    ! litter heterotrophic respiration (gC/day/m**2 of total ground)
    REAL(r_std),DIMENSION(kjpindex,nvegtypes)       :: resp_hetero_litter
    ! soil heterotrophic respiration (gC/day/m**2 of total ground)
    REAL(r_std),DIMENSION(kjpindex,nvegtypes)       :: resp_hetero_soil
    INTEGER(i_std) :: iyear
    ! for deforestation data reading loop
    INTEGER(i_std) :: jyear
    ! to be returned by IOIPSL
    INTEGER(i_std)                                  :: year, month, day   
    REAL(r_std)                                     :: sec
    ! calendar stuff not provided by IOIPSL
    INTEGER(i_std)                                  :: month_len
    LOGICAL                                        :: year_bissex
  
    REAL(r_std),DIMENSION(nbp_glo) :: clay_g
    REAL(r_std),DIMENSION(nbp_glo) :: space_nat_g
    REAL(r_std),ALLOCATABLE,DIMENSION(:,:,:,:) :: soilcarbon_input_g
    REAL(r_std),ALLOCATABLE,DIMENSION(:,:,:)   :: control_moist_g
    REAL(r_std),ALLOCATABLE,DIMENSION(:,:,:)   :: control_temp_g
    REAL(r_std),ALLOCATABLE,DIMENSION(:,:)     :: npp_equil_g
  
    !---------------------------------------------------------------------
    ! first of all: store time step in common value
    itime = kjit
    CALL itau2ymds(itime, dtradia, year, month, day, sec)
    ! ask for the calendar
    CALL ioget_calendar(one_year, one_day)

    month_len = ioget_mon_len (year,month)

    ! Height of vegetation must be initialized to height_x at each time step
    DO j=1,npft
       height(:,j)=height_x(:,j+1)
    ENDDO
    z_soil(0) = 0.
    z_soil(1:nbdl) = diaglev(1:nbdl)
    DO j=1,npft
       rprof(:,j) = 1./humcste(ipft_sechiba(j))
    ENDDO
    !-
    ! 1 do initialisation
    !-
    resp_growth_x=0
    IF (l_first_stomate) THEN
       IF (long_print) THEN
          WRITE (numout,*) ' l_first_stomate : call stomate_init'
       ENDIF
       !
       ! 1.1 allocation, file definitions. Set flags.
       !
       CALL stomate_init (kjpij, kjpindex, index, ldforcing_write, lalo, &
            rest_id_stom, hist_id_stom)

       co2_flux_monthly(:) = 0.0
       !
       ! 1.2 read PFT data
       !
       CALL data (kjpindex, lalo)
       !
       ! 1.3 read STOMATE's start file
       !
       CALL readstart &
            &    (kjpindex, index, lalo, resolution, &
            day_counter, dt_days_read, date, &
            ind, adapted, regenerate, &
            humrel_daily, litterhum_daily, &
            t2m_daily, t2m_min_daily, tsurf_daily, tsoil_daily, &
            soilhum_daily, precip_daily, &
            gpp_daily, npp_daily, turnover_daily, &
            humrel_month, humrel_week, t2m_longterm, &
            tlong_ref, t2m_month, t2m_week, &
            tsoil_month, soilhum_month, fireindex, firelitter, &
            maxhumrel_lastyear, maxhumrel_thisyear, &
            minhumrel_lastyear, minhumrel_thisyear, &
            maxgppweek_lastyear, maxgppweek_thisyear, &
            gdd0_lastyear, gdd0_thisyear, &
            precip_lastyear, precip_thisyear, &
            gdd_m5_dormance, gdd_midwinter, ncd_dormance, ngd_minus5, &
            PFTpresent, npp_longterm, lm_lastyearmax, lm_thisyearmax, &
            maxfpc_lastyear, maxfpc_thisyear, &
            turnover_longterm, gpp_week, biomass, resp_maint_part, &
            fvm, fv, leaf_age, leaf_frac, &
            senescence, when_growthinit, age, &
            resp_hetero, resp_maint, resp_growth, co2_fire, co2_to_bm_dgvm, &
            veget_lastlight, everywhere, need_adjacent, &
            RIP_time, time_lowgpp, &
            time_hum_min, hum_min_dormance, &
            litterpart, litter, dead_leaves, &
            carbon, black_carbon, lignin_struc,turnover_time,&
            prod10,prod100,flux10, flux100)
       ! deforestation variables added as arguments

       ! 1.4 read the boundary conditions
       !
       CALL readbc (kjpindex, lalo, resolution, tlong_ref)
       !
       ! 1.5 check time step
       !
       ! 1.5.1 allow STOMATE's time step to change
       !       although this is dangerous
       !
       dt_days = dt_slow/one_day
       IF (dt_days /= dt_days_read) THEN
          WRITE(numout,*) 'slow_processes: STOMATE time step changes:', &
               dt_days_read,' -> ',dt_days
       ENDIF

       ! 1.5.2 time step has to be a multiple of a full day

       IF ( ( dt_days-REAL(NINT(dt_days),r_std) ) > min_stomate ) THEN
          WRITE(numout,*) 'slow_processes: STOMATE time step is wrong:', &
               &                    dt_days,' days.'
          STOP
       ENDIF

       ! 1.5.3 upper limit to STOMATE's time step

       IF ( dt_days > max_dt_days ) THEN
          WRITE(numout,*) 'slow_processes: STOMATE time step too large:', &
               &                    dt_days,' days.'
          STOP
       ENDIF

       ! 1.5.4 STOMATE time step must not be less than the forcing time step

       IF ( dtradia > dt_days*one_day ) THEN
          WRITE(numout,*) &
               &   'slow_processes: STOMATE time step smaller than forcing time step.'
          STOP
       ENDIF

       ! 1.5.5 some more messages

       WRITE(numout,*) 'slow_processes, STOMATE time step (d): ', dt_days

       !
       ! 1.6 write forcing file for stomate?
       !
       IF (ldforcing_write) THEN

          !Config  Key  = STOMATE_FORCING_NAME
          !Config  Desc = Name of STOMATE's forcing file
          !Config  Def  = NONE
          !Config  Help = Name that will be given
          !Config         to STOMATE's offline forcing file
          !-
          forcing_name = stomate_forcing_name              ! compatibilit avec driver Nicolas
          CALL getin_p('STOMATE_FORCING_NAME',forcing_name)

          IF ( TRIM(forcing_name) /= 'NONE' ) THEN  
             IF (is_root_prc) CALL SYSTEM ('rm -f '//TRIM(forcing_name))
             WRITE(numout,*) 'writing a forcing file for STOMATE.'

             !Config  Key  = STOMATE_FORCING_MEMSIZE
             !Config  Desc = Size of STOMATE forcing data in memory (MB)
             !Config  Def  = 50
             !Config  Help = This variable determines how many
             !Config         forcing states will be kept in memory.
             !Config         Must be a compromise between memory
             !Config         use and frequeny of disk access.

             max_totsize = 50
             CALL getin_p('STOMATE_FORCING_MEMSIZE', max_totsize)      
             max_totsize = max_totsize*1000000

             totsize_1step = &
                  &      SIZE(clay)*KIND(clay) &
                  &     +SIZE(humrel_daily_x)*KIND(humrel_daily_x) &
                  &     +SIZE(litterhum_daily)*KIND(litterhum_daily) &
                  &     +SIZE(t2m_daily)*KIND(t2m_daily) &
                  &     +SIZE(t2m_min_daily)*KIND(t2m_min_daily) &
                  &     +SIZE(tsurf_daily)*KIND(tsurf_daily) &
                  &     +SIZE(tsoil_daily)*KIND(tsoil_daily) &
                  &     +SIZE(soilhum_daily)*KIND(soilhum_daily) &
                  &     +SIZE(precip_daily)*KIND(precip_daily) &
                  &     +SIZE(gpp_daily_x)*KIND(gpp_daily_x) &
                  &     +SIZE(resp_maint_part_x)*KIND(resp_maint_part_x) &
                  &     +SIZE(veget_x)*KIND(veget_x) &
                  &     +SIZE(veget_max_x)*KIND(veget_max_x) &
                  &     +SIZE(lai_x)*KIND(lai_x)

             CALL reduce_sum(totsize_1step,totsize_tmp)
             CALL bcast(totsize_tmp)
             totsize_1step=totsize_tmp 
             ! total number of forcing steps
             nsft = INT(one_year/(dt_slow/one_day))

             ! number of forcing steps in memory
             nsfm = MIN(nsft, &
                &       MAX(1,NINT( REAL(max_totsize,r_std) &
                &                  /REAL(totsize_1step,r_std))))

             CALL init_forcing (kjpindex,nsfm,nsft)

             isf(:) = (/ (i,i=1,nsfm) /)
             nf_written(:) = .FALSE.
             nf_cumul(:) = 0

             iisf = 0

             !-
             IF (is_root_prc) THEN
                ier = NF90_CREATE (TRIM(forcing_name),NF90_SHARE,forcing_id)
                ier = NF90_PUT_ATT (forcing_id,NF90_GLOBAL,'dtradia',dtradia)
                ier = NF90_PUT_ATT (forcing_id,NF90_GLOBAL,'dt_slow',dt_slow)
                ier = NF90_PUT_ATT (forcing_id,NF90_GLOBAL, &
                     &                            'nsft',REAL(nsft,r_std))
                ier = NF90_PUT_ATT (forcing_id,NF90_GLOBAL, &
                     &                            'kjpij',REAL(iim_g*jjm_g,r_std))
                ier = NF90_PUT_ATT (forcing_id,NF90_GLOBAL, &
                     &                            'kjpindex',REAL(nbp_glo,r_std))
                !-
                ier = NF90_DEF_DIM (forcing_id,'points',nbp_glo,d_id(1))
                ier = NF90_DEF_DIM (forcing_id,'layers',nbdl,d_id(2))
                ier = NF90_DEF_DIM (forcing_id,'pft',nvm,d_id(3))
                direct=2
                ier = NF90_DEF_DIM (forcing_id,'direction',direct,d_id(4))
                nneigh=8
                ier = NF90_DEF_DIM (forcing_id,'nneigh',nneigh,d_id(5))
                ier = NF90_DEF_DIM (forcing_id,'time',nsft,d_id(6))
                ier = NF90_DEF_DIM (forcing_id,'nbparts',nparts,d_id(7))
                !-
                ier = NF90_DEF_VAR (forcing_id,'points',    r_typ,d_id(1),vid)
                ier = NF90_DEF_VAR (forcing_id,'layers',    r_typ,d_id(2),vid)
                ier = NF90_DEF_VAR (forcing_id,'pft',       r_typ,d_id(3),vid)
                ier = NF90_DEF_VAR (forcing_id,'direction', r_typ,d_id(4),vid)
                ier = NF90_DEF_VAR (forcing_id,'nneigh',    r_typ,d_id(5),vid)
                ier = NF90_DEF_VAR (forcing_id,'time',      r_typ,d_id(6),vid)
                ier = NF90_DEF_VAR (forcing_id,'nbparts',   r_typ,d_id(7),vid)
                ier = NF90_DEF_VAR (forcing_id,'index',     r_typ,d_id(1),vid)
                ier = NF90_DEF_VAR (forcing_id,'contfrac',  r_typ,d_id(1),vid) 
                ier = NF90_DEF_VAR (forcing_id,'lalo', &
                     &                            r_typ,(/ d_id(1),d_id(4) /),vid)
                ier = NF90_DEF_VAR (forcing_id,'neighbours', &
                     &                            r_typ,(/ d_id(1),d_id(5) /),vid)
                ier = NF90_DEF_VAR (forcing_id,'resolution', &
                     &                            r_typ,(/ d_id(1),d_id(4) /),vid)
                ier = NF90_DEF_VAR (forcing_id,'clay', &
                     &                            r_typ,(/ d_id(1),d_id(6) /),vid)
                ier = NF90_DEF_VAR (forcing_id,'humrel', &
                     &                            r_typ,(/ d_id(1),d_id(3),d_id(6) /),vid)
                ier = NF90_DEF_VAR (forcing_id,'litterhum', &
                     &                            r_typ,(/ d_id(1),d_id(6) /),vid)
                ier = NF90_DEF_VAR (forcing_id,'t2m', &
                     &                            r_typ,(/ d_id(1),d_id(6) /),vid)
                ier = NF90_DEF_VAR (forcing_id,'t2m_min', &
                     &                            r_typ,(/ d_id(1),d_id(6) /),vid)
                ier = NF90_DEF_VAR (forcing_id,'tsurf', &
                     &                            r_typ,(/ d_id(1),d_id(6) /),vid)
                ier = NF90_DEF_VAR (forcing_id,'tsoil', &
                     &                            r_typ,(/ d_id(1),d_id(2),d_id(6) /),vid)
                ier = NF90_DEF_VAR (forcing_id,'soilhum', &
                     &                            r_typ,(/ d_id(1),d_id(2),d_id(6) /),vid)
                ier = NF90_DEF_VAR (forcing_id,'precip', &
                     &                            r_typ,(/ d_id(1),d_id(6) /),vid)
                ier = NF90_DEF_VAR (forcing_id,'gpp', &
                     &                            r_typ,(/ d_id(1),d_id(3),d_id(6) /),vid)
                ier = NF90_DEF_VAR (forcing_id,'veget', &
                     &                            r_typ,(/ d_id(1),d_id(3),d_id(6) /),vid)
                ier = NF90_DEF_VAR (forcing_id,'veget_max', &
                     &                            r_typ,(/ d_id(1),d_id(3),d_id(6) /),vid)
                ier = NF90_DEF_VAR (forcing_id,'lai', &
                     &                            r_typ,(/ d_id(1),d_id(3),d_id(6) /),vid)
                ier = NF90_DEF_VAR (forcing_id,'resp_maint_part', &
                     &                       r_typ,(/ d_id(1),d_id(3),d_id(6),d_id(7) /),vid)
                ier = NF90_ENDDEF (forcing_id)
                !-
                ier = NF90_INQ_VARID (forcing_id,'points',vid)
                ier = NF90_PUT_VAR (forcing_id,vid, &
                     &                            (/(REAL(i,r_std),i=1,nbp_glo) /))
                ier = NF90_INQ_VARID (forcing_id,'layers',vid)
                ier = NF90_PUT_VAR (forcing_id,vid,(/(REAL(i,r_std),i=1,nbdl)/))
                ier = NF90_INQ_VARID (forcing_id,'pft',vid)
                ier = NF90_PUT_VAR (forcing_id,vid,(/(REAL(i,r_std),i=1,nvm)/))
                ier = NF90_INQ_VARID (forcing_id,'direction',vid)
                ier = NF90_PUT_VAR (forcing_id,vid,(/(REAL(i,r_std),i=1,2)/))
                ier = NF90_INQ_VARID (forcing_id,'nneigh',vid)
                ier = NF90_PUT_VAR (forcing_id,vid,(/(REAL(i,r_std),i=1,8)/))
                ier = NF90_INQ_VARID (forcing_id,'time',vid)
                ier = NF90_PUT_VAR (forcing_id,vid,(/(REAL(i,r_std),i=1,nsft)/))
                ier = NF90_INQ_VARID (forcing_id,'nbparts',vid)
                ier = NF90_PUT_VAR (forcing_id,vid,(/(REAL(i,r_std),i=1,nparts)/))
                ier = NF90_INQ_VARID (forcing_id,'index',vid)  
                ier = NF90_PUT_VAR (forcing_id,vid,REAL(index_g,r_std))
                ier = NF90_INQ_VARID (forcing_id,'contfrac',vid)
                ier = NF90_PUT_VAR (forcing_id,vid,REAL(contfrac_g,r_std))
                ier = NF90_INQ_VARID (forcing_id,'lalo',vid)
                ier = NF90_PUT_VAR (forcing_id,vid,lalo_g)
                !ym attention a neighbours, a modifier plus tard      
                ier = NF90_INQ_VARID (forcing_id,'neighbours',vid)
                ier = NF90_PUT_VAR (forcing_id,vid,REAL(neighbours_g,r_std))
                ier = NF90_INQ_VARID (forcing_id,'resolution',vid)
                ier = NF90_PUT_VAR (forcing_id,vid,resolution_g)
             ENDIF
          ENDIF
       ENDIF
       !
       ! 1.7 write forcing file for the soil?
       !
       IF (ldcarbon_write) THEN
          !
          !Config  Key  = STOMATE_CFORCING_NAME
          !Config  Desc = Name of STOMATE's carbon forcing file
          !Config  Def  = NONE
          !Config  Help = Name that will be given to STOMATE's carbon
          !Config         offline forcing file
          !-
          Cforcing_name = stomate_Cforcing_name               ! compatibilit avec driver Nicolas
          CALL getin_p('STOMATE_CFORCING_NAME',Cforcing_name)

          IF ( TRIM(Cforcing_name) /= 'NONE' ) THEN  
             IF (is_root_prc) CALL SYSTEM ('rm -f '//TRIM(Cforcing_name))
             !
             ! time step of forcesoil
             !
             !Config  Key  = FORCESOIL_STEP_PER_YEAR
             !Config  Desc = Number of time steps per year for carbon spinup.
             !Config  Def  = 12
             !Config  Help = Number of time steps per year for carbon spinup.
             nparan = 12
             CALL getin_p('FORCESOIL_STEP_PER_YEAR', nparan)
  
             IF ( nparan < 1 ) nparan = 1

             !Config  Key  = FORCESOIL_NB_YEAR
             !Config  Desc = Number of years saved for carbon spinup.
             !Config  Def  = 1
             !Config  Help = Number of years saved for carbon spinup.
             nbyear=1
             CALL getin_p('FORCESOIL_NB_YEAR', nbyear)
             dt_forcesoil = 0.
             nparan = nparan+1
             DO WHILE (dt_forcesoil < dt_slow/one_day)
                nparan = nparan-1
                IF (nparan < 1) THEN
                   STOP 'Problem 1 with number of soil forcing time steps.'
                ENDIF
                dt_forcesoil = one_year/REAL(nparan,r_std)
             ENDDO

             IF ( nparan > nparanmax ) THEN
                STOP 'Problem 2 with number of soil forcing time steps.'
             ENDIF

             WRITE(numout,*) 'time step of soil forcing (d): ',dt_forcesoil

             ALLOCATE(times(0:nparan))
             times(:) = (/ ((REAL(i,r_std)*dt_forcesoil),i=0,nparan*nbyear) /)
  
             ALLOCATE( nforce(nparan*nbyear))
             nforce(:) = 0

             ALLOCATE(control_moist(kjpindex,nlevs,nparan*nbyear))
             ALLOCATE(npp_equil(kjpindex,nparan*nbyear))
             ALLOCATE(npp_tot(kjpindex))
             ALLOCATE(control_temp(kjpindex,nlevs,nparan*nbyear))
             ALLOCATE(soilcarbon_input(kjpindex,ncarb,nvegtypes,nparan*nbyear))

             npp_equil(:,:) = zero
             npp_tot(:) = zero
             control_moist(:,:,:) = zero
             control_temp(:,:,:) = zero
             soilcarbon_input(:,:,:,:) = zero
          ENDIF
       ENDIF
       !
       ! 1.8 calculate STOMATE's vegetation fractions
       !     from veget_x, veget_max_x, fv, and fvm
       !
       CALL stomate_vegconvert (kjpindex,'in',fraction_nobio, &
            veget_max_x,veget_x,veget_max,veget,fvm,fv)
       !
       ! 1.9 initialize some variables
       !     STOMATE diagnoses some variables for SECHIBA :
       !       assim_param, deadleaf_cover, etc.
       !     These variables can be recalculated easily
       !     from STOMATE's prognostic variables.
       !     height is saved in Sechiba.
       !
       IF (control%ok_stomate) THEN
          CALL stomate_var_init &
               &      (kjpindex, veget, veget_max, leaf_age, leaf_frac, &
               &       tlong_ref, t2m_month, dead_leaves, &
               &       veget_x, lai_x, qsintmax, deadleaf_cover, assim_param_x,&
               &       prod10, prod100, flux10, flux100, &
               &       convflux,cflux_prod10, cflux_prod100)
          ! deforestation variables added as arguments
       ENDIF
       !
       ! 1.10 reset flag
       !
       l_first_stomate = .FALSE.
       !
       ! 1.11 return
       !
       RETURN
    ENDIF  ! first call
    IF (bavard >= 4) THEN
       WRITE(numout,*) 'DATE ',date,' ymds', year, month, day, sec, '-- stp --', itime, do_slow
    ENDIF
    !-
    ! 2 prepares restart file for the next simulation
    !-
    IF (ldrestart_write) THEN
       IF (long_print) THEN
          WRITE (numout,*) &
               &      ' we have to complete restart file with STOMATE variables'
       ENDIF
       CALL writerestart &
            &   (kjpindex, index, &
            &    day_counter, dt_days, date, &
            &    ind, adapted, regenerate, &
            &    humrel_daily, litterhum_daily, &
            &    t2m_daily, t2m_min_daily, tsurf_daily, tsoil_daily, &
            &    soilhum_daily, precip_daily, &
            &    gpp_daily, npp_daily, turnover_daily, &
            &    humrel_month, humrel_week, t2m_longterm, &
            &    tlong_ref, t2m_month, t2m_week, &
            &    tsoil_month, soilhum_month, fireindex, firelitter, &
            &    maxhumrel_lastyear, maxhumrel_thisyear, &
            &    minhumrel_lastyear, minhumrel_thisyear, &
            &    maxgppweek_lastyear, maxgppweek_thisyear, &
            &    gdd0_lastyear, gdd0_thisyear, &
            &    precip_lastyear, precip_thisyear, &
            &    gdd_m5_dormance, gdd_midwinter, ncd_dormance, ngd_minus5, &
            &    PFTpresent, npp_longterm, lm_lastyearmax, lm_thisyearmax, &
            &    maxfpc_lastyear, maxfpc_thisyear, &
            &    turnover_longterm, gpp_week, biomass, resp_maint_part, &
            &    fvm, fv, leaf_age, leaf_frac, &
            &    senescence, when_growthinit, age, &
            &    resp_hetero, resp_maint, resp_growth, co2_fire, co2_to_bm_dgvm, &
            &    veget_lastlight, everywhere, need_adjacent, &
            &    RIP_time, time_lowgpp, &
            &    time_hum_min, hum_min_dormance, &
            &    litterpart, litter, dead_leaves, &
            &    carbon, black_carbon, lignin_struc,turnover_time,&
            &    prod10,prod100,flux10, flux100)
       ! deforestation variables added as arguments 

       IF (ldforcing_write .AND. TRIM(forcing_name) /= 'NONE' ) THEN  
          CALL forcing_write(forcing_id,1,iisf)
          !
          IF (is_root_prc) ier = NF90_CLOSE (forcing_id)
          forcing_id=-1
       ENDIF

       IF (ldcarbon_write .AND. TRIM(Cforcing_name) /= 'NONE' ) THEN  
          WRITE(numout,*) &
               &      'stomate: writing the forcing file for carbon spinup'
          !
          DO iatt = 1, nparan*nbyear
             IF ( nforce(iatt) > 0 ) THEN
                soilcarbon_input(:,:,:,iatt) = &
                     &          soilcarbon_input(:,:,:,iatt)/REAL(nforce(iatt),r_std)
                control_moist(:,:,iatt) = &
                     &          control_moist(:,:,iatt)/REAL(nforce(iatt),r_std)
                control_temp(:,:,iatt) = &
                     &          control_temp(:,:,iatt)/REAL(nforce(iatt),r_std)
                npp_equil(:,iatt) = &
                     &          npp_equil(:,iatt)/REAL(nforce(iatt),r_std)
             ELSE
                WRITE(numout,*) &
                     &         'We have no soil carbon forcing data for this time step:', &
                     &         iatt
                WRITE(numout,*) ' -> we set them to zero'
                soilcarbon_input(:,:,:,iatt) = zero
                control_moist(:,:,iatt) = zero
                control_temp(:,:,iatt) = zero
                npp_equil(:,iatt) = zero
             ENDIF
          ENDDO

          IF (is_root_prc) THEN
             ALLOCATE(soilcarbon_input_g(nbp_glo,ncarb,nvegtypes,nparan*nbyear))
             ALLOCATE(control_moist_g(nbp_glo,nlevs,nparan*nbyear))
             ALLOCATE(control_temp_g(nbp_glo,nlevs,nparan*nbyear))
             ALLOCATE(npp_equil_g(nbp_glo,nparan*nbyear))
          ENDIF
  
          CALL gather(clay,clay_g)
          CALL gather(space_nat,space_nat_g)
          CALL gather(soilcarbon_input,soilcarbon_input_g)
          CALL gather(control_moist,control_moist_g)
          CALL gather(control_temp,control_temp_g)
          CALL gather(npp_equil,npp_equil_g)
  
          !-
          IF (is_root_prc) THEN
             ier = NF90_CREATE (TRIM(Cforcing_name),NF90_WRITE,Cforcing_id)
             ier = NF90_PUT_ATT (Cforcing_id,NF90_GLOBAL, &
                  &                        'kjpindex',REAL(nbp_glo,r_std))
             ier = NF90_PUT_ATT (Cforcing_id,NF90_GLOBAL, &
                  &                        'nparan',REAL(nparan,r_std))
             ier = NF90_PUT_ATT (Cforcing_id,NF90_GLOBAL, &
                  &                        'nbyear',REAL(nbyear,r_std))
             ier = NF90_DEF_DIM (Cforcing_id,'points',nbp_glo,d_id(1))
             ier = NF90_DEF_DIM (Cforcing_id,'carbtype',ncarb,d_id(2))
             ier = NF90_DEF_DIM (Cforcing_id,'vegtype',nvegtypes,d_id(3))
             ier = NF90_DEF_DIM (Cforcing_id,'level',nlevs,d_id(4))
             ier = NF90_DEF_DIM (Cforcing_id,'time_step',nparan*nbyear,d_id(5))
             !-
             ier = NF90_DEF_VAR (Cforcing_id,'points',    r_typ,d_id(1),vid)
             ier = NF90_DEF_VAR (Cforcing_id,'carbtype',  r_typ,d_id(2),vid)
             ier = NF90_DEF_VAR (Cforcing_id,'vegtype',   r_typ,d_id(3),vid)
             ier = NF90_DEF_VAR (Cforcing_id,'level',     r_typ,d_id(4),vid)
             ier = NF90_DEF_VAR (Cforcing_id,'time_step', r_typ,d_id(5),vid)
             ier = NF90_DEF_VAR (Cforcing_id,'index',     r_typ,d_id(1),vid)
             ier = NF90_DEF_VAR (Cforcing_id,'clay',      r_typ,d_id(1),vid)
             ier = NF90_DEF_VAR (Cforcing_id,'space_nat', r_typ,d_id(1),vid)
             ier = NF90_DEF_VAR (Cforcing_id,'soilcarbon_input',r_typ, &
                  &                        (/ d_id(1),d_id(2),d_id(3),d_id(5) /),vid)
             ier = NF90_DEF_VAR (Cforcing_id,'control_moist',r_typ, &
                  &                        (/ d_id(1),d_id(4),d_id(5) /),vid)
             ier = NF90_DEF_VAR (Cforcing_id,'control_temp',r_typ, &
                  &                        (/ d_id(1),d_id(4),d_id(5) /),vid)
             ier = NF90_DEF_VAR (Cforcing_id,'npp_equil',r_typ, &
                  &                        (/ d_id(1),d_id(5) /),vid)
             ier = NF90_ENDDEF (Cforcing_id)
             !-
             ier = NF90_INQ_VARID (Cforcing_id,'points',vid)
             ier = NF90_PUT_VAR (Cforcing_id,vid, &
                  &                          (/(REAL(i,r_std),i=1,nbp_glo)/))
             ier = NF90_INQ_VARID (Cforcing_id,'carbtype',vid)
             ier = NF90_PUT_VAR (Cforcing_id,vid, &
                  &                        (/(REAL(i,r_std),i=1,ncarb)/))
             ier = NF90_INQ_VARID (Cforcing_id,'vegtype',vid)
             ier = NF90_PUT_VAR (Cforcing_id,vid, &
                  &                        (/(REAL(i,r_std),i=1,nvegtypes)/))
             ier = NF90_INQ_VARID (Cforcing_id,'level',vid)
             ier = NF90_PUT_VAR (Cforcing_id,vid, &
                  &                          (/(REAL(i,r_std),i=1,nlevs)/))
             ier = NF90_INQ_VARID (Cforcing_id,'time_step',vid)
             ier = NF90_PUT_VAR (Cforcing_id,vid, &
                  &                          (/(REAL(i,r_std),i=1,nparan*nbyear)/))
             ier = NF90_INQ_VARID (Cforcing_id,'index',vid)
             ier = NF90_PUT_VAR (Cforcing_id,vid, REAL(index_g,r_std) )
             ier = NF90_INQ_VARID (Cforcing_id,'clay',vid)
             ier = NF90_PUT_VAR (Cforcing_id,vid, clay_g )
             ier = NF90_INQ_VARID (Cforcing_id,'space_nat',vid)
             ier = NF90_PUT_VAR (Cforcing_id,vid, space_nat_g )
             ier = NF90_INQ_VARID (Cforcing_id,'soilcarbon_input',vid)
             ier = NF90_PUT_VAR (Cforcing_id,vid, soilcarbon_input_g )
             ier = NF90_INQ_VARID (Cforcing_id,'control_moist',vid)
             ier = NF90_PUT_VAR (Cforcing_id,vid, control_moist_g )
             ier = NF90_INQ_VARID (Cforcing_id,'control_temp',vid)
             ier = NF90_PUT_VAR (Cforcing_id,vid, control_temp_g )
             ier = NF90_INQ_VARID (Cforcing_id,'npp_equil',vid)
             ier = NF90_PUT_VAR (Cforcing_id,vid, npp_equil_g )
             !-
             ier = NF90_CLOSE (Cforcing_id)
             Cforcing_id = -1
          ENDIF
  
          IF (is_root_prc) THEN
             DEALLOCATE(soilcarbon_input_g)
             DEALLOCATE(control_moist_g)
             DEALLOCATE(control_temp_g)
             DEALLOCATE(npp_equil_g)
          ENDIF
       ENDIF
       RETURN
    ENDIF  ! write restart-files
    !-
    ! 3 Here's where the serious things begin -
    !   Check whether stomate and dgvm have to be called
    !-
    !
    ! 3.1 update day counter
    !
    day_counter = day_counter+dtradia


    IF (NINT(day_counter) >= NINT(dt_slow)) THEN
       !
       ! 3.2 we have to call STOMATE
       !     (or daily update of vegetation characteristics)?
       !
       ! reset counter
       day_counter = zero
       do_slow = .TRUE.
       !
       ! 3.3 is one year over?
       !     EndOfYear must be true once per year
       !     during a call of stomate_season.
       !
       ! increase date
       dt_days = dt_slow/one_day

       ! compilation bug : nint added 
       date = date+nint(dt_days)

       IF (     ((date-NINT(date/one_year)*one_year) <  .0 ) &
            &      .AND.((date-NINT(date/one_year)*one_year) >= -dt_days) ) THEN
          EndOfYear = .TRUE.

          IF ( (bavard >= 2).AND.EndOfYear.AND.do_slow) THEN
             WRITE(numout,*) 'stomate: EndOfYear'
          ENDIF
       ELSE
          EndOfYear = .FALSE.

       ENDIF

    ELSE
       do_slow = .FALSE.
       EndOfYear = .FALSE.

    ENDIF
    IF ( (ldcarbon_write) .AND. TRIM(Cforcing_name) /= 'NONE' &
         &    .AND.((date-NINT(date/one_year)*one_year) == dt_days) ) THEN
       control_moist(:,:,:) = zero
       control_temp(:,:,:) = zero
       soilcarbon_input(:,:,:,:) = zero
       nforce=0
    ENDIF
    !
    ! 4 Special treatment for some input arrays.
    !
    !
    ! 4.1 Sum of liquid and solid precipitation
    !
    precip = ( precip_rain+precip_snow )*one_day/dtradia
    !
    ! 4.2 Transform from dimension nvm to dimension npft.
    !     In SECHIBA, some variables are defined
    !       for all PFTs AND on bare ground -> nvm
    !     In STOMATE/LPJ, we do not treat bare ground.
    !       The same variable need not be
    !       defined on bare ground in STOMATE/LPJ  -> npft
    !     Moreover, several Stomate-PFTs can be aggregated
    !       to one single PFT in Sechiba.
    !     This is not used for the moment, but it might be used
    !       once an age structure is introduced in Stomate.
    !       In that case, the different age groups of a PFT may
    !       be treated separately in Stomate,
    !       but as a single PFT in Sechiba.
    !     The corresponding variables exchanged with the rest
    !       of SECHIBA are tagged _x.
    !     XXX_x(:,ibare_sechiba) is the bare soil part.
    !     ipft_sechiba(j) is the Sechiba PFT index corresponding
    !       to the Stomate PFT index j.
    !
    ! 4.2.1 calculate STOMATE's vegetation fractions
    !       from veget_x, veget_max_x, fv, and fvm
    !
    CALL stomate_vegconvert (kjpindex,'in',fraction_nobio, &
         veget_max_x,veget_x,veget_max,veget,fvm,fv)
    DO j=1,npft

       ! 4.2.2 GPP
       ! gpp in gC/m**2 of total ground/day
       WHERE (veget_max_x(:,ipft_sechiba(j)) > 0.0)
          gpp(:,j) = gpp_x(:,ipft_sechiba(j))*one_day/dtradia &
               &              *(veget_max(:,j)/veget_max_x(:,ipft_sechiba(j)))
       ELSEWHERE
          gpp(:,j) = 0.0
       ENDWHERE

       ! 4.2.3 moisture availability
       humrel(:,j) = humrel_x(:,ipft_sechiba(j))

    ENDDO

    space_nat(:) = 1.0
    DO j=1,npft
       IF (.NOT.natural(j) ) THEN
          space_nat(:) = space_nat(:)-veget_max(:,j)
       ENDIF
    ENDDO
    !
    ! 5 "daily" variables
    !   Note: If dt_days /= 1, then xx_daily are not daily variables,
    !         but that is not really a problem.
    !
    !
    ! 5.1 accumulate instantaneous variables
    !     and eventually calculate daily mean value
    !
    CALL stomate_accu (kjpindex, npft, dt_slow, dtradia, &
         &                   do_slow, humrel, humrel_daily)
    CALL stomate_accu (kjpindex,    1, dt_slow, dtradia, &
         &                   do_slow, litterhumdiag, litterhum_daily)
    CALL stomate_accu (kjpindex,    1, dt_slow, dtradia, &
         &                   do_slow, t2m, t2m_daily)
    CALL stomate_accu (kjpindex,    1, dt_slow, dtradia, &
         &                   do_slow, temp_sol, tsurf_daily)
    CALL stomate_accu (kjpindex, nbdl, dt_slow, dtradia, &
         &                   do_slow, stempdiag, tsoil_daily)
    CALL stomate_accu (kjpindex, nbdl, dt_slow, dtradia, &
         &                   do_slow, shumdiag, soilhum_daily)
    CALL stomate_accu (kjpindex,    1, dt_slow, dtradia, &
         &                   do_slow, precip, precip_daily)
    CALL stomate_accu (kjpindex, npft, dt_slow, dtradia, &
         &                   do_slow, gpp, gpp_daily)
    !
    ! 5.2 daily minimum temperature
    !
    t2m_min_daily(:) = MIN( t2m_min(:), t2m_min_daily(:) )
    !
    ! 5.3 calculate respiration (NV 14/5/2002)
    !
    ! 5.3.1 calculate maintenance respiration
    !
    CALL maint_respiration &
         &  (kjpindex,dtradia,t2m,tlong_ref,stempdiag,height,veget_max,space_nat, &
         &   rprof,biomass,resp_maint_part_radia)

    resp_maint_radia(:,:) = zero
    DO j=1,npft
       DO k= 1, nparts
          resp_maint_radia(:,j) = resp_maint_radia(:,j) &
               &                           +resp_maint_part_radia(:,j,k)
       ENDDO
    ENDDO
    resp_maint_part(:,:,:)= resp_maint_part(:,:,:) &
         &                       +resp_maint_part_radia(:,:,:)
    !
    ! 5.3.2 the whole litter stuff:
    !       litter update, lignin content, PFT parts, litter decay,
    !       litter heterotrophic respiration, dead leaf soil cover.
    !        No vertical discretisation in the soil for litter decay.
    !
    turnover_littercalc = turnover_daily * dtradia/one_day
    bm_to_littercalc    = bm_to_litter*dtradia/one_day

    CALL littercalc (kjpindex, dtradia/one_day, space_nat, &
         turnover_littercalc, bm_to_littercalc, &
         temp_sol, stempdiag, shumdiag, litterhumdiag, &
         litterpart, litter, dead_leaves, lignin_struc, &
         deadleaf_cover, resp_hetero_litter, &
         soilcarbon_input_inst, control_temp_inst, control_moist_inst)
    resp_hetero_litter=resp_hetero_litter*dtradia/one_day
    !
    ! 5.3.3 soil carbon dynamics: heterotrophic respiration from the soil.
    !       For the moment, no vertical discretisation.
    !       We might later introduce a vertical discretisation.
    !       However, in that case, we would have to treat the vertical
    !       exchanges of carbon between the different levels.
    !
    CALL soilcarbon (kjpindex, dtradia/one_day, clay, space_nat, &
         soilcarbon_input_inst, control_temp_inst, control_moist_inst, &
         carbon, resp_hetero_soil)
    resp_hetero_soil=resp_hetero_soil*dtradia/one_day
    resp_hetero_radia = resp_hetero_litter+resp_hetero_soil
    resp_hetero= resp_hetero+resp_hetero_radia
    !
    ! 6 Daily processes
    !
    IF (do_slow) THEN

       ! update veget_max_new and space_nat_new by reading data file
       defor = .FALSE.

       CALL getin_p('DEFOR', defor)

       IF(defor) then

          IF(EndOfYear) then

             IF (is_root_prc) THEN
                open(10,file='veget_defor.dat')     
                read(10,*)jyear,veget_max_new
             ENDIF
             CALL bcast(jyear)
             CALL bcast(veget_max_new)

             space_nat_new(:) = 1.0

             DO j = 1, npft
                IF ( .NOT. natural(j) ) THEN
                   space_nat_new(:) = space_nat_new(:) - veget_max_new(:,j)
                ENDIF
             ENDDO
             CALL natagritot (kjpindex, ito_natagri, space_nat_new, veget_max_new)

          ENDIF
       ENDIF
       !
       ! 6.1 total natural space
       !
       !
       ! 6.2 Calculate longer-term "meteorological" and biological parameters
       !
       CALL season &
            &    (kjpindex, dt_days, EndOfYear, space_nat, &
            &     veget, veget_max, &
            &     humrel_daily, t2m_daily, tsoil_daily, soilhum_daily, &
            &     precip_daily, npp_daily, biomass, &
            &     turnover_daily, gpp_daily, when_growthinit, &
            &     maxhumrel_lastyear, maxhumrel_thisyear, &
            &     minhumrel_lastyear, minhumrel_thisyear, &
            &     maxgppweek_lastyear, maxgppweek_thisyear, &
            &     gdd0_lastyear, gdd0_thisyear, &
            &     precip_lastyear, precip_thisyear, &
            &     lm_lastyearmax, lm_thisyearmax, &
            &     maxfpc_lastyear, maxfpc_thisyear, &
            &     humrel_month, humrel_week, t2m_longterm, &
            &     tlong_ref, t2m_month, t2m_week, tsoil_month, soilhum_month, &
            &     npp_longterm, turnover_longterm, gpp_week, &
            &     gdd_m5_dormance, gdd_midwinter, ncd_dormance, ngd_minus5, &
            &     time_lowgpp, time_hum_min, hum_min_dormance, herbivores)
       !
       ! 6.3 transform GPP from gC/(m**2 of total ground)/day to
       !     gC/(m**2 of nat/agri ground)/day
       !
       CALL natagritot (kjpindex, ito_natagri, space_nat, gpp_daily)
       !
       ! 6.4 STOMATE: allocation, phenology, etc.
       !
       IF (control%ok_stomate) THEN

          ! 6.4.1.a transform spatial fractions from fraction
          !       of total space to fraction of natural/agricultural space

          CALL natagritot (kjpindex, ito_natagri, space_nat, veget)
          CALL natagritot (kjpindex, ito_natagri, space_nat, veget_max)

          ! 6.4.1.b update lai
          IF (control%ok_pheno) THEN ! lai from stomate
             DO j = 1, npft
                WHERE ( veget_max(:,j) .GT. min_sechiba )
                   lai(:,j) = biomass(:,j,ileaf)/veget_max(:,j)*sla(j)
                ELSEWHERE
                   lai(:,j) = 0.0
                ENDWHERE
             ENDDO
          ELSE
             CALL  setlai(kjpindex,lai) ! lai prescribed
          ENDIF

          ! 6.4.2 call stomate

          CALL StomateLpj &
               &      (kjpindex, dt_days, EndOfYear, &
               &       neighbours, resolution, space_nat, &
               &       clay, herbivores, &
               &       tsurf_daily, tsoil_daily, t2m_daily, t2m_min_daily, &
               &       litterhum_daily, soilhum_daily, &
               &       maxhumrel_lastyear, minhumrel_lastyear, &
               &       gdd0_lastyear, precip_lastyear, &
               &       humrel_month, humrel_week, tlong_ref, t2m_month, t2m_week, &
               &       tsoil_month, soilhum_month, &
               &       gdd_m5_dormance, gdd_midwinter, ncd_dormance, ngd_minus5, &
               &       turnover_longterm, gpp_daily, time_lowgpp, &
               &       time_hum_min, maxfpc_lastyear, resp_maint_part,&
               &       PFTpresent, age, fireindex, firelitter, &
               &       leaf_age, leaf_frac, biomass, ind, adapted, regenerate, &
               &       senescence, when_growthinit, litterpart, litter, &
               &       dead_leaves, carbon, black_carbon, lignin_struc, &
               &       veget_max, veget, npp_longterm, lm_lastyearmax, &
               &       veget_lastlight, everywhere, need_adjacent, RIP_time, &
               &       lai, rprof,npp_daily, turnover_daily, turnover_time,&
               &       control_moist_inst, control_temp_inst, soilcarbon_input_inst, &
               &       co2_to_bm_dgvm, co2_fire, resp_hetero, resp_maint, &
               &       resp_growth, height, deadleaf_cover, vcmax, vjmax, &
               &       t_photo_min, t_photo_opt, t_photo_max,bm_to_litter,&
               &       prod10, prod100, flux10, flux100, space_nat_new, veget_max_new,&
               &       convflux, cflux_prod10, cflux_prod100, defor)
          ! deforestation variables added as arguments + EndOfYear

          ! 6.4.3 transform spatial fractions from fraction
          !       of natural/agricultural space to fraction of total space

          CALL natagritot (kjpindex, ito_total, space_nat, veget)
          CALL natagritot (kjpindex, ito_total, space_nat, veget_max)
          !
          ! 6.5 output: transform from dimension npft to dimension nvm
          !     Several Stomate-PFTs may correspond
          !       to a single Sechiba-PFT (see 4.2).
          !     We sum up the vegetation cover over these Stomate-PFTs.
          !     Mean LAI, height, and Vmax is calculated
          !       by ponderating with (maximum) vegetation cover.
          !
          ! 6.5.1 calculate veget_x, veget_max_x,
          !         fv and fvm from veget and veget_max
          !
          CALL stomate_vegconvert &
               &      (kjpindex,'out',fraction_nobio, &
               &       veget_max_x,veget_x,veget_max,veget,fvm,fv)

          ! 6.5.2 lai and height

          CALL stomate_var_xout (kjpindex,lai,veget_max,zero,lai_x)
          CALL stomate_var_xout (kjpindex,height,veget_max,zero,height_x)

          ! 6.5.3 photosynthesis parameters

          CALL stomate_var_xout &
               &      (kjpindex,vcmax,veget,zero,assim_param_x(:,:,ivcmax))
          CALL stomate_var_xout &
               &      (kjpindex,vjmax,veget,zero,assim_param_x(:,:,ivjmax))

          CALL stomate_var_xout &
               &      (kjpindex,t_photo_min,veget,zero,assim_param_x(:,:,itmin))
          CALL stomate_var_xout &
               &      (kjpindex,t_photo_opt,veget,zero,assim_param_x(:,:,itopt))
          CALL stomate_var_xout &
               &      (kjpindex,t_photo_max,veget,zero,assim_param_x(:,:,itmax))
          !
          ! 6.6 update forcing variables for soil carbon
          !
          IF (ldcarbon_write  .AND. TRIM(Cforcing_name) /= 'NONE') THEN
             !
             ! determime the carbon soil time step we are falling into
             !
             ! where are we within the current year?
             sf_time = date &
                &           -FLOOR(date/(one_year*REAL(nbyear,r_std)))*one_year*nbyear
             DO WHILE ( sf_time > one_year )
                sf_time = sf_time-one_year
             ENDDO
             DO WHILE ( sf_time < 0. )
                sf_time = sf_time+one_year
             ENDDO
             iyear=1
             IF (iatt > nparan) iatt=1
             IF (    (times(iatt-1) > sf_time ) &
                  &          .OR.(times(iatt)   <= sf_time ) ) THEN
                iatt = nparan
                ! look for corresponding time step
                DO i = nparan, 1, -1
                   IF ( times(i) > sf_time ) iatt = i
                ENDDO
             ENDIF ! otherwise, iatt was already OK!

             ! we know now what soil forcing time step we are talking about.
             ! Increase counter for this soil carbon time step
             !   and update "mean" forcing variables.

             nforce(iatt) = nforce(iatt)+1

             soilcarbon_input(:,:,:,iatt+(iyear-1)*nparan) = &
                  &        soilcarbon_input(:,:,:,iatt+(iyear-1)*nparan) &
                  &       +soilcarbon_input_inst(:,:,:)
             control_moist(:,:,iatt+(iyear-1)*nparan) = &
                  &        control_moist(:,:,iatt+(iyear-1)*nparan) &
                  &       +control_moist_inst(:,:)
             control_temp(:,:,iatt+(iyear-1)*nparan) = &
                  &        control_temp(:,:,iatt+(iyear-1)*nparan) &
                  &       +control_temp_inst(:,:)
             npp_equil(:,iatt+(iyear-1)*nparan) = &
                  &        npp_equil(:,iatt+(iyear-1)*nparan) &
                  &       +npp_tot(:)
             !     nforce(iatt) = 1
             !     soilcarbon_input(:,:,:,iatt) = soilcarbon_input_inst(:,:,:)
             !     control_moist(:,:,iatt) = control_moist_inst(:,:)
             !     control_temp(:,:,iatt) = control_temp_inst(:,:)
             !     npp_equil(:,iatt) = npp_tot(:)
          ENDIF
          !
          ! 6.7 updates qsintmax
          !
          qsintmax(:,:) = qsintcst*veget_x(:,:)*lai_x(:,:)
       ENDIF
       !
       ! 6.8 write forcing file?
       !     ldforcing_write should only be .TRUE.
       !       if STOMATE is run in coupled mode.
       !     In stand-alone mode, the forcing file is read!
       !
       IF ( ldforcing_write .AND. TRIM(forcing_name) /= 'NONE' ) THEN
          CALL natagritot (kjpindex, ito_total, space_nat, gpp_daily)
          gpp_daily_x(:,:) = 0.0
          resp_maint_part_x(:,:,:) = 0.0
          DO j = 1, npft
             ! don't worry about weightings. humrel is not modified in STOMATE.
             humrel_daily_x(:,ipft_sechiba(j)) = humrel_daily(:,j)
             gpp_daily_x(:,ipft_sechiba(j)) = &
                  &        gpp_daily_x(:,ipft_sechiba(j)) &
                  &       +gpp_daily(:,j)*dt_slow/one_day
             resp_maint_part_x(:,ipft_sechiba(j),:) = &
                  &        resp_maint_part_x(:,ipft_sechiba(j),:) &
                  &       +resp_maint_part(:,j,:)*dt_slow/one_day
          ENDDO
          !
          ! bare soil moisture availability has not been treated
          !   in STOMATE (doesn't matter)
          !
          humrel_daily_x(:,ibare_sechiba) = humrel_x(:,ibare_sechiba)

          ! next forcing step in memory
          iisf = iisf+1

          ! how many times have we treated this forcing state
          xn = REAL(nf_cumul(isf(iisf)),r_std)

          ! cumulate. be careful :
          !   precipitation is multiplied by dt_slow/one_day
          IF (cumul_forcing) THEN
             clay_fm(:,iisf) = (xn*clay_fm(:,iisf)+clay(:))/(xn+1.)
             humrel_daily_x_fm(:,:,iisf) = &
                  &        (xn*humrel_daily_x_fm(:,:,iisf)+humrel_daily_x(:,:))/(xn+1.)
             litterhum_daily_fm(:,iisf) = &
                  &        (xn*litterhum_daily_fm(:,iisf)+litterhum_daily(:))/(xn+1.)
             t2m_daily_fm(:,iisf) = &
                  &        (xn*t2m_daily_fm(:,iisf)+t2m_daily(:))/(xn+1.)
             t2m_min_daily_fm(:,iisf) = &
                  &        (xn*t2m_min_daily_fm(:,iisf)+t2m_min_daily(:))/(xn+1.)
             tsurf_daily_fm(:,iisf) = &
                  &        (xn*tsurf_daily_fm(:,iisf)+tsurf_daily(:))/(xn+1.)
             tsoil_daily_fm(:,:,iisf) = &
                  &        (xn*tsoil_daily_fm(:,:,iisf)+tsoil_daily(:,:))/(xn+1.)
             soilhum_daily_fm(:,:,iisf) = &
                  &        (xn*soilhum_daily_fm(:,:,iisf)+soilhum_daily(:,:))/(xn+1.)
             precip_fm(:,iisf) = &
                  &        (xn*precip_fm(:,iisf)+precip_daily(:)*dt_slow/one_day)/(xn+1.)
             gpp_daily_x_fm(:,:,iisf) = &
                  &        (xn*gpp_daily_x_fm(:,:,iisf)+gpp_daily_x(:,:))/(xn+1.)
             resp_maint_part_x_fm(:,:,:,iisf) = &
                  &        ( xn*resp_maint_part_x_fm(:,:,:,iisf) &
                  &         +resp_maint_part_x(:,:,:) )/(xn+1.)
             veget_x_fm(:,:,iisf) = &
                  &        (xn*veget_x_fm(:,:,iisf)+veget_x(:,:) )/(xn+1.)
             veget_max_x_fm(:,:,iisf) = &
                  &        (xn*veget_max_x_fm(:,:,iisf)+veget_max_x(:,:) )/(xn+1.)
             lai_x_fm(:,:,iisf) = &
                  &        (xn*lai_x_fm(:,:,iisf)+lai_x(:,:) )/(xn+1.)
          ELSE
             clay_fm(:,iisf) = clay(:)
             humrel_daily_x_fm(:,:,iisf) = humrel_daily_x(:,:)
             litterhum_daily_fm(:,iisf) = +litterhum_daily(:)
             t2m_daily_fm(:,iisf) = t2m_daily(:)
             t2m_min_daily_fm(:,iisf) =t2m_min_daily(:)
             tsurf_daily_fm(:,iisf) = tsurf_daily(:)
             tsoil_daily_fm(:,:,iisf) =tsoil_daily(:,:)
             soilhum_daily_fm(:,:,iisf) =soilhum_daily(:,:)
             precip_fm(:,iisf) = precip_daily(:)
             gpp_daily_x_fm(:,:,iisf) =gpp_daily_x(:,:)
             resp_maint_part_x_fm(:,:,:,iisf) = resp_maint_part_x(:,:,:)
             veget_x_fm(:,:,iisf) = veget_x(:,:)
             veget_max_x_fm(:,:,iisf) =veget_max_x(:,:)
             lai_x_fm(:,:,iisf) =lai_x(:,:)
          ENDIF
          nf_cumul(isf(iisf)) = nf_cumul(isf(iisf))+1

          ! do we have to write the forcing states?
          IF (iisf == nsfm) THEN

             ! write these forcing states
             CALL forcing_write(forcing_id,1,nsfm)
             ! determine which forcing states must be read
             isf(1) = isf(nsfm)+1
             IF ( isf(1) > nsft ) isf(1) = 1
             DO iisf = 2, nsfm
                isf(iisf) = isf(iisf-1)+1
                IF (isf(iisf) > nsft)  isf(iisf) = 1
             ENDDO

             ! read them
!for debug only!
!             CALL forcing_read(forcing_id,nsfm)

             iisf = 0

          ENDIF

       ENDIF
       ! 6.9 compute daily co2_flux
       resp_auto_tot(:) = 0.0

       DO j=1,npft
          resp_auto_tot(:) = resp_auto_tot(:) &
               &                      +resp_maint(:,j)+resp_growth(:,j)
       ENDDO

       ! total photosynthesis (in gC/m**2/day)
       gpp_tot(:) = 0.0
       DO j=1,npft
          gpp_tot(:) = gpp_tot(:)+gpp_daily(:,j)
       ENDDO

       ! CO2 flux in gC/m**2/sec
       !   (positive towards the atmosphere) is sum of:
       ! 1/ heterotrophic respiration from natural and agricultural ground
       ! 2/ maintenance respiration from the plants
       ! 3/ growth respiration from the plants
       ! 4/ co2 created by fire
       ! 5/ - co2 taken up in the DGVM to establish saplings.
       ! 6/ - co2 taken up by photosyntyhesis

       co2_flux_daily(:) = ( resp_hetero(:,inat) &
            &                       +resp_hetero(:,iagri) &
            &                       +resp_auto_tot(:) &
            &                       +co2_fire(:)-co2_to_bm_dgvm(:)-gpp_tot(:) )
       !
       co2_flux_monthly(:) = co2_flux_monthly(:) + co2_flux_daily(:)
       IF   (  (day .EQ. month_len) .AND. (INT(sec) .EQ. 0) )  THEN
          IF ( control%ok_stomate ) THEN
             CALL histwrite (hist_id_stomate, 'CO2FLUX_MONTHLY', itime, &
                  co2_flux_monthly, kjpindex, hori_index)
          ENDIF
          co2_flux_monthly(:) = co2_flux_monthly(:)* &
               resolution(:,1)*resolution(:,2)*contfrac(:)
          net_co2_flux_monthly = sum(co2_flux_monthly) 

          WRITE(numout,*) 'net_co2_flux_monthly = ',net_co2_flux_monthly 
  
          CALL reduce_sum(net_co2_flux_monthly,net_co2_flux_monthly_sum)
          IF (is_root_prc) THEN
             OPEN( unit=39,              &
                  file="stomate_co2flux.data", &
                  action="write",              &
                  position="append",           &
                  iostat=ios  )
             IF ( ios /= 0 ) THEN
                STOP "Erreur lors de la lecture/ecriture du fichier stomate_co2flux.data"
             ELSE
                WRITE(numout,*)
                WRITE(numout,*) "Ecriture du fichier stomate_co2flux.data"
                WRITE(numout,*)
             END IF
             WRITE(39,*) net_co2_flux_monthly_sum*1e-15
             CLOSE( unit=39 )
          ENDIF
          co2_flux_monthly(:) = 0.0
       ENDIF
       !
       ! 6.10 reset daily variables
       !
       humrel_daily(:,:) = 0.0
       litterhum_daily(:) = 0.0
       t2m_daily(:) = 0.0
       t2m_min_daily(:) = large_value
       tsurf_daily(:) = 0.0
       tsoil_daily(:,:) = 0.0
       soilhum_daily(:,:) = 0.0
       precip_daily(:) = 0.0
       gpp_daily(:,:) = 0.0
       resp_maint_part(:,:,:)=0.0
       resp_hetero=0.0
       IF (bavard >= 3) THEN
          WRITE(numout,*) 'stomate_main: daily processes done'
       ENDIF

    ENDIF  ! daily processes?
    ! CO2FLUX Daily values are saved each dtradia,
    ! then the value is wrong for the first day and
    ! shifted for other days.
    IF ( hist_id > 0 ) THEN
       CALL histwrite (hist_id, 'CO2FLUX', itime, &
            co2_flux_daily, kjpindex, index)
    ENDIF
    IF ( hist2_id > 0 ) THEN
       CALL histwrite (hist2_id, 'CO2FLUX', itime, &
            co2_flux_daily, kjpindex, index)
    ENDIF

    !
    ! 7 Outputs from Stomate
    !   co2_flux is assigned a value only if STOMATE is activated.
    !   Otherwise, the calling hydrological module must do this itself.
    !
    IF ( control%ok_stomate ) THEN

       ! total autotrophic respiration
       resp_auto_tot(:) = 0.0

       DO j = 1, npft
          resp_auto_tot(:) = resp_auto_tot(:) &
               &                      +resp_maint_radia(:,j) &
               &                      +resp_growth(:,j)*dtradia/one_day
          resp_maint_x(:,j+1) = resp_maint_radia(:,j)
          resp_growth_x(:,j+1)= resp_growth(:,j)*dtradia/one_day
       ENDDO
       resp_hetero_x=resp_hetero_radia(:,inat)+resp_hetero_radia(:,iagri)
       resp_maint_x(:,1) = 0.0
       ! total photosynthesis (in gC/m**2/day)
       gpp_tot(:) = 0.0
       DO jv=1,nvm
          gpp_tot(:) = gpp_tot(:)+gpp_x(:,jv)
       ENDDO

       ! CO2 flux in gC/m**2/sec (positive towards the atmosphere) is sum of:
       ! 1/ heterotrophic respiration from natural and agricultural ground
       ! 2/ maintenance respiration from the plants
       ! 3/ growth respiration from the plants
       ! 4/ co2 created by fire
       ! 5/ - co2 taken up in the DGVM to establish saplings.
       ! 6/ - co2 taken up by photosyntyhesis

       co2_flux(:) = resp_hetero_radia(:,inat) &
            &               +resp_hetero_radia(:,iagri) &
            &               +(co2_fire(:)-co2_to_bm_dgvm(:))*dtradia/one_day &
            &               +resp_auto_tot(:)-gpp_tot(:)

    ENDIF
    !
    ! 8 message
    !
    IF ( (bavard >= 2).AND.EndOfYear.AND.do_slow) THEN
       WRITE(numout,*) 'stomate: EndOfYear'
    ENDIF
    IF (bavard >= 4) WRITE(numout,*) 'Leaving stomate_main'
    IF (long_print) WRITE (numout,*) ' stomate_main done '
    !--------------------------
  END SUBROUTINE stomate_main
!
!=
!
SUBROUTINE stomate_init &
 &  (kjpij, kjpindex, index, ldforcing_write, lalo, &
 &   rest_id_stom, hist_id_stom)
!---------------------------------------------------------------------
! interface description
! input scalar
! Total size of the un-compressed grid
  INTEGER(i_std),INTENT(in)                   :: kjpij
! Domain size
  INTEGER(i_std),INTENT(in)                   :: kjpindex
! Logical for _forcing_ file to write
  LOGICAL,INTENT(in)                          :: ldforcing_write
! Geogr. coordinates (latitude,longitude) (degrees)
  REAL(r_std),DIMENSION(kjpindex,2),INTENT(in) :: lalo
! STOMATE's _Restart_ file file identifier
  INTEGER(i_std),INTENT(in)                   :: rest_id_stom
! STOMATE's _history_ file file identifier
  INTEGER(i_std),INTENT(in)                   :: hist_id_stom
! input fields
! Indeces of the points on the map
  INTEGER(i_std),DIMENSION(kjpindex),INTENT(in) :: index
! local declaration
  REAL(r_std)                                  :: tmp_day(1)
! soil depth taken for canopy
  REAL(r_std)                                  :: zcanop
! soil depths at diagnostic levels
  REAL(r_std),DIMENSION(nbdl)                  :: zsoil
! Index
  INTEGER(i_std)                              :: l
! allocation error
  LOGICAL                                     :: l_error
! Global world fraction of vegetation type map
  REAL(r_std),DIMENSION(360,180,nvm)           :: veget_ori_on_disk
  INTEGER(i_std)                              :: ier
! indices
  INTEGER(i_std)                              :: ji,j,ipd
!---------------------------------------------------------------------
!
! 1 online diagnostics
!   (by default, "bavard" is set to 1 in stomate_constants)
!
  !Config  Key  = BAVARD
  !Config  Desc = level of online diagnostics in STOMATE (0-4)
  !Config  Def  = 1
  !Config  Help = With this variable, you can determine
  !Config         how much online information STOMATE
  !Config         gives during the run. 0 means
  !Config         virtually no info.
!
  bavard = 1
  CALL getin_p('BAVARD', bavard)

  IF ( kjpindex > 0 ) THEN
  !
    !Config  Key  = STOMATE_DIAGPT
    !Config  Desc = Index of grid point for online diagnostics
    !Config  Def  = 1
    !Config  Help = This is the index of the grid point which
  !               will be used for online diagnostics.
    ipd = 1
    CALL getin_p('STOMATE_DIAGPT',ipd)
    ipd = MIN( ipd, kjpindex )
    WRITE(numout,*) 'Stomate: '
    WRITE(numout,*) '  Index of grid point for online diagnostics: ',ipd
    WRITE(numout,*) '  Lon, lat:',lalo(ipd,2),lalo(ipd,1)
    WRITE(numout,*) '  Index of this point on GCM grid: ',index(ipd)
  !
  ENDIF
!
! 2 check consistency of flags
!
  IF ( ( .NOT. control%ok_stomate ) .AND. control%ok_dgvm ) THEN
    WRITE(numout,*) 'Cannot do dynamical vegetation without STOMATE.'
    WRITE(numout,*) 'We stop.'
      STOP
  ENDIF

  IF ((.NOT.control%ok_co2).AND.control%ok_stomate) THEN
    WRITE(numout,*) 'Cannot call STOMATE without GPP.'
    WRITE(numout,*) 'We stop.'
      STOP
  ENDIF

  IF ( ( .NOT. control%ok_co2 ) .AND. ldforcing_write ) THEN
    WRITE(numout,*) &
 &    'Cannot write forcing file if photosynthesis is not activated'
    WRITE(numout,*) 'We stop.'
    STOP
  ENDIF
!
! 3 messages
!
  WRITE(numout,*) 'stomate: first call'
  WRITE(numout,*) '  Photosynthesis: ', control%ok_co2
  WRITE(numout,*) '  STOMATE: ', control%ok_stomate
  WRITE(numout,*) '  LPJ: ', control%ok_dgvm
!
! 4 allocation of STOMATE's variables
!
  l_error = .FALSE.
  ALLOCATE(veget_max(kjpindex,npft),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(space_nat(kjpindex),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(ind(kjpindex,npft),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(adapted(kjpindex,npft),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(regenerate(kjpindex,npft),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(humrel_daily(kjpindex,npft),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(litterhum_daily(kjpindex),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(t2m_daily(kjpindex),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(t2m_min_daily(kjpindex),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(tsurf_daily(kjpindex),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(tsoil_daily(kjpindex,nbdl),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(soilhum_daily(kjpindex,nbdl),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(precip_daily(kjpindex),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(gpp_daily(kjpindex,npft),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(npp_daily(kjpindex,npft),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(turnover_daily(kjpindex,npft,nparts),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(turnover_littercalc(kjpindex,npft,nparts),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(humrel_month(kjpindex,npft),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(humrel_week(kjpindex,npft),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(t2m_longterm(kjpindex),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(tlong_ref(kjpindex),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(t2m_month(kjpindex),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(t2m_week(kjpindex),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(tsoil_month(kjpindex,nbdl),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(soilhum_month(kjpindex,nbdl),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(fireindex(kjpindex,nvegtypes),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(firelitter(kjpindex,nvegtypes),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(maxhumrel_lastyear(kjpindex,npft),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(maxhumrel_thisyear(kjpindex,npft),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(minhumrel_lastyear(kjpindex,npft),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(minhumrel_thisyear(kjpindex,npft),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(maxgppweek_lastyear(kjpindex,npft),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(maxgppweek_thisyear(kjpindex,npft),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(gdd0_lastyear(kjpindex),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(gdd0_thisyear(kjpindex),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(precip_lastyear(kjpindex),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(precip_thisyear(kjpindex),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(gdd_m5_dormance(kjpindex,npft),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(gdd_midwinter(kjpindex,npft),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(ncd_dormance(kjpindex,npft),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(ngd_minus5(kjpindex,npft),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(PFTpresent(kjpindex,npft),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(npp_longterm(kjpindex,npft),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(lm_lastyearmax(kjpindex,npft),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(lm_thisyearmax(kjpindex,npft),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(maxfpc_lastyear(kjpindex,npft),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(maxfpc_thisyear(kjpindex,npft),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(turnover_longterm(kjpindex,npft,nparts),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(gpp_week(kjpindex,npft),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(biomass(kjpindex,npft,nparts),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(senescence(kjpindex,npft),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(when_growthinit(kjpindex,npft),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(age(kjpindex,npft),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(resp_hetero(kjpindex,nvegtypes),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(resp_hetero_radia(kjpindex,nvegtypes),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(resp_maint(kjpindex,npft),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(resp_growth(kjpindex,npft),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(co2_fire(kjpindex),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(co2_to_bm_dgvm(kjpindex),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(veget_lastlight(kjpindex,npft),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(everywhere(kjpindex,npft),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(need_adjacent(kjpindex,npft),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(leaf_age(kjpindex,npft,nleafages),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(leaf_frac(kjpindex,npft,nleafages),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(RIP_time(kjpindex,npft),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(time_lowgpp(kjpindex,npft),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(time_hum_min(kjpindex,npft),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(hum_min_dormance(kjpindex,npft),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(fvm(kjpindex,npft),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(fv(kjpindex,npft),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(litterpart(kjpindex,npft,nlitt),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(litter(kjpindex,nlitt,nvegtypes,nlevs),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(dead_leaves(kjpindex,npft,nlitt),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(carbon(kjpindex,ncarb,nvegtypes),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(black_carbon(kjpindex),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(lignin_struc(kjpindex,nvegtypes,nlevs),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(turnover_time(kjpindex,npft),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(co2_flux_daily(kjpindex),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(co2_flux_monthly(kjpindex),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(bm_to_litter(kjpindex,npft,nparts),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(bm_to_littercalc(kjpindex,npft,nparts),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(herbivores(kjpindex,npft),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(hori_index(kjpindex),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(horipft_index(kjpindex*npft),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(resp_maint_part_radia(kjpindex,npft,nparts),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(resp_maint_radia(kjpindex,npft),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(resp_maint_part(kjpindex,npft,nparts),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  resp_maint_part(:,:,:)=zero

! allocation for deforestation variables
  ALLOCATE (veget_max_new(kjpindex,npft), stat=ier)
  l_error = l_error .OR. (ier.NE.0)
  ALLOCATE (space_nat_new(kjpindex), stat=ier)
  l_error = l_error .OR. (ier.NE.0)
  ALLOCATE (horip10_index(kjpindex*10), stat=ier)
  l_error = l_error .OR. (ier.NE.0)
  ALLOCATE (horip100_index(kjpindex*100), stat=ier)
  l_error = l_error .OR. (ier.NE.0)
  ALLOCATE (horip11_index(kjpindex*11), stat=ier)
  l_error = l_error .OR. (ier.NE.0)
  ALLOCATE (horip101_index(kjpindex*101), stat=ier)
  l_error = l_error .OR. (ier.NE.0)
  ALLOCATE (prod10(kjpindex,0:10), stat=ier)
  l_error = l_error .OR. (ier.NE.0)
  ALLOCATE (prod100(kjpindex,0:100), stat=ier)
  l_error = l_error .OR. (ier.NE.0)
  ALLOCATE (flux10(kjpindex,10), stat=ier)
  l_error = l_error .OR. (ier.NE.0)
  ALLOCATE (flux100(kjpindex,100), stat=ier)
  l_error = l_error .OR. (ier.NE.0)
  ALLOCATE (convflux(kjpindex), stat=ier)
  l_error = l_error .OR. (ier.NE.0)
  ALLOCATE (cflux_prod10(kjpindex), stat=ier)
  l_error = l_error .OR. (ier.NE.0)
  ALLOCATE (cflux_prod100(kjpindex), stat=ier)
  l_error = l_error .OR. (ier.NE.0)
!
  IF (l_error) THEN
      STOP 'stomate_init: error in memory allocation'
  ENDIF
!
! 5 file definitions: stored in common variables
!
  hist_id_stomate = hist_id_stom
  rest_id_stomate = rest_id_stom
  hori_index(:) = index(:)

! Get the indexing table for the vegetation fields.
! In STOMATE we work on
! reduced grids but to store in the full 3D filed vegetation variable
! we need another index table : indexpft

  DO j = 1, npft
    DO ji = 1, kjpindex
      horipft_index((j-1)*kjpindex+ji) = index(ji)+(j-1)*kjpij
    ENDDO
  ENDDO

! indexing tables added for deforestation fields
  DO j = 1, 10
    DO ji = 1, kjpindex
      horip10_index((j-1)*kjpindex+ji) = index(ji)+(j-1)*kjpij
    ENDDO
  ENDDO

  DO j = 1, 100
    DO ji = 1, kjpindex
      horip100_index((j-1)*kjpindex+ji) = index(ji)+(j-1)*kjpij
    ENDDO
  ENDDO

  DO j = 1, 11
    DO ji = 1, kjpindex
      horip11_index((j-1)*kjpindex+ji) = index(ji)+(j-1)*kjpij
    ENDDO
  ENDDO

  DO j = 1, 101
    DO ji = 1, kjpindex
      horip101_index((j-1)*kjpindex+ji) = index(ji)+(j-1)*kjpij
    ENDDO
  ENDDO
!
! 6 some flags
!
!
  !Config  Key  = HERBIVORES
  !Config  Desc = herbivores allowed?
  !Config  Def  = n
  !Config  Help = With this variable, you can determine
  !Config         if herbivores are activated
!
  ok_herbivores = .FALSE.
  CALL getin_p('HERBIVORES', ok_herbivores)
!
  WRITE(numout,*) 'herbivores activated: ',ok_herbivores
!
  !Config  Key  = TREAT_EXPANSION
  !Config  Desc = treat expansion of PFTs across a grid cell?
  !Config  Def  = n
  !Config  Help = With this variable, you can determine
  !Config         whether we treat expansion of PFTs across a
  !Config         grid cell.
!
  treat_expansion = .FALSE.
  CALL getin_p('TREAT_EXPANSION', treat_expansion)
!
  WRITE(numout,*) &
 &  'expansion across a grid cell is treated: ',treat_expansion

!
  ! 7 some global initializations
  !
  bm_to_litter(:,:,:) = zero
  turnover_daily(:,:,:) = zero
  resp_hetero(:,:) = zero
  co2_flux_daily(:) = zero
  co2_flux_monthly(:) = zero
  
  
  ! initialisation of land cover change variables
  prod10(:,:)  = zero
  prod100(:,:) = zero
  flux10(:,:)  = zero
  flux100(:,:) = zero
  convflux(:)  = zero
  cflux_prod10(:) = zero
  cflux_prod100(:)= zero
!--------------------------
END SUBROUTINE stomate_init
!
!=
!
  SUBROUTINE stomate_clear
!---------------------------------------------------------------------
! 1. Deallocate all dynamics variables
  IF (ALLOCATED(veget_max)) DEALLOCATE(veget_max)
  IF (ALLOCATED(space_nat)) DEALLOCATE(space_nat)
  IF (ALLOCATED(ind)) DEALLOCATE(ind)
  IF (ALLOCATED(adapted)) DEALLOCATE(adapted)
  IF (ALLOCATED(regenerate)) DEALLOCATE(regenerate)
  IF (ALLOCATED(humrel_daily)) DEALLOCATE(humrel_daily)
  IF (ALLOCATED(litterhum_daily)) DEALLOCATE(litterhum_daily)
  IF (ALLOCATED(t2m_daily))  DEALLOCATE(t2m_daily)
  IF (ALLOCATED(t2m_min_daily))  DEALLOCATE(t2m_min_daily)
  IF (ALLOCATED(tsurf_daily))  DEALLOCATE(tsurf_daily)
  IF (ALLOCATED(tsoil_daily)) DEALLOCATE(tsoil_daily)
  IF (ALLOCATED(soilhum_daily)) DEALLOCATE(soilhum_daily)
  IF (ALLOCATED(precip_daily)) DEALLOCATE(precip_daily)
  IF (ALLOCATED(gpp_daily)) DEALLOCATE(gpp_daily)
  IF (ALLOCATED(npp_daily)) DEALLOCATE(npp_daily)
  IF (ALLOCATED(turnover_daily)) DEALLOCATE(turnover_daily)
  IF (ALLOCATED(turnover_littercalc)) DEALLOCATE(turnover_littercalc)
  IF (ALLOCATED(humrel_month)) DEALLOCATE(humrel_month)
  IF (ALLOCATED(humrel_week)) DEALLOCATE(humrel_week)
  IF (ALLOCATED(t2m_longterm)) DEALLOCATE(t2m_longterm)
  IF (ALLOCATED(tlong_ref)) DEALLOCATE(tlong_ref)
  IF (ALLOCATED(t2m_month)) DEALLOCATE(t2m_month)
  IF (ALLOCATED(t2m_week)) DEALLOCATE(t2m_week)
  IF (ALLOCATED(tsoil_month)) DEALLOCATE(tsoil_month)
  IF (ALLOCATED(soilhum_month)) DEALLOCATE(soilhum_month)
  IF (ALLOCATED(fireindex)) DEALLOCATE(fireindex)
  IF (ALLOCATED(firelitter)) DEALLOCATE(firelitter)
  IF (ALLOCATED(maxhumrel_lastyear)) DEALLOCATE(maxhumrel_lastyear)
  IF (ALLOCATED(maxhumrel_thisyear)) DEALLOCATE(maxhumrel_thisyear)
  IF (ALLOCATED(minhumrel_lastyear)) DEALLOCATE(minhumrel_lastyear)
  IF (ALLOCATED(minhumrel_thisyear)) DEALLOCATE(minhumrel_thisyear)
  IF (ALLOCATED(maxgppweek_lastyear)) DEALLOCATE(maxgppweek_lastyear)
  IF (ALLOCATED(maxgppweek_thisyear)) DEALLOCATE(maxgppweek_thisyear)
  IF (ALLOCATED(gdd0_lastyear)) DEALLOCATE(gdd0_lastyear)
  IF (ALLOCATED(gdd0_thisyear)) DEALLOCATE(gdd0_thisyear)
  IF (ALLOCATED(precip_lastyear)) DEALLOCATE(precip_lastyear)
  IF (ALLOCATED(precip_thisyear)) DEALLOCATE(precip_thisyear)
  IF (ALLOCATED(gdd_m5_dormance)) DEALLOCATE(gdd_m5_dormance)
  IF (ALLOCATED(gdd_midwinter)) DEALLOCATE(gdd_midwinter)
  IF (ALLOCATED(ncd_dormance)) DEALLOCATE(ncd_dormance)
  IF (ALLOCATED(ngd_minus5))  DEALLOCATE(ngd_minus5)
  IF (ALLOCATED(PFTpresent)) DEALLOCATE(PFTpresent)
  IF (ALLOCATED(npp_longterm)) DEALLOCATE(npp_longterm)
  IF (ALLOCATED(lm_lastyearmax)) DEALLOCATE(lm_lastyearmax)
  IF (ALLOCATED(lm_thisyearmax)) DEALLOCATE(lm_thisyearmax)
  IF (ALLOCATED(maxfpc_lastyear)) DEALLOCATE(maxfpc_lastyear)
  IF (ALLOCATED(maxfpc_thisyear)) DEALLOCATE(maxfpc_thisyear)
  IF (ALLOCATED(turnover_longterm)) DEALLOCATE(turnover_longterm)
  IF (ALLOCATED(gpp_week)) DEALLOCATE(gpp_week)
  IF (ALLOCATED(biomass)) DEALLOCATE(biomass)
  IF (ALLOCATED(senescence)) DEALLOCATE(senescence)
  IF (ALLOCATED(when_growthinit)) DEALLOCATE(when_growthinit)
  IF (ALLOCATED(age))  DEALLOCATE(age)
  IF (ALLOCATED(resp_hetero)) DEALLOCATE(resp_hetero)
  IF (ALLOCATED(resp_hetero_radia)) DEALLOCATE(resp_hetero_radia)
  IF (ALLOCATED(resp_maint)) DEALLOCATE(resp_maint)
  IF (ALLOCATED(resp_growth)) DEALLOCATE(resp_growth)
  IF (ALLOCATED(co2_fire)) DEALLOCATE(co2_fire)
  IF (ALLOCATED(co2_to_bm_dgvm)) DEALLOCATE(co2_to_bm_dgvm)
  IF (ALLOCATED(veget_lastlight)) DEALLOCATE(veget_lastlight)
  IF (ALLOCATED(everywhere)) DEALLOCATE(everywhere)
  IF (ALLOCATED(need_adjacent)) DEALLOCATE(need_adjacent)
  IF (ALLOCATED(leaf_age)) DEALLOCATE(leaf_age)
  IF (ALLOCATED(leaf_frac)) DEALLOCATE(leaf_frac)
  IF (ALLOCATED(RIP_time)) DEALLOCATE(RIP_time)
  IF (ALLOCATED(time_lowgpp)) DEALLOCATE(time_lowgpp)
  IF (ALLOCATED(time_hum_min)) DEALLOCATE(time_hum_min)
  IF (ALLOCATED(hum_min_dormance)) DEALLOCATE(hum_min_dormance)
  IF (ALLOCATED(fvm)) DEALLOCATE(fvm)
  IF (ALLOCATED(fv)) DEALLOCATE(fv)
  IF (ALLOCATED(litterpart)) DEALLOCATE(litterpart)
  IF (ALLOCATED(litter)) DEALLOCATE(litter)
  IF (ALLOCATED(dead_leaves)) DEALLOCATE(dead_leaves)
  IF (ALLOCATED(carbon)) DEALLOCATE(carbon)
  IF (ALLOCATED(black_carbon)) DEALLOCATE(black_carbon)
  IF (ALLOCATED(lignin_struc)) DEALLOCATE(lignin_struc)
  IF (ALLOCATED(turnover_time)) DEALLOCATE(turnover_time)
  IF (ALLOCATED(co2_flux_daily)) DEALLOCATE(co2_flux_daily)
  IF (ALLOCATED(co2_flux_monthly)) DEALLOCATE(co2_flux_monthly)
  IF (ALLOCATED(bm_to_litter)) DEALLOCATE(bm_to_litter)
  IF (ALLOCATED(bm_to_littercalc)) DEALLOCATE(bm_to_littercalc)
  IF (ALLOCATED(herbivores)) DEALLOCATE(herbivores)
  IF (ALLOCATED(resp_maint_part_radia)) DEALLOCATE(resp_maint_part_radia)
  IF (ALLOCATED(resp_maint_radia)) DEALLOCATE(resp_maint_radia)
  IF (ALLOCATED(resp_maint_part)) DEALLOCATE(resp_maint_part)
  IF (ALLOCATED(hori_index)) DEALLOCATE(hori_index)
  IF (ALLOCATED(horipft_index)) DEALLOCATE(horipft_index)
  IF (ALLOCATED(clay_fm)) DEALLOCATE(clay_fm)
  IF (ALLOCATED(humrel_daily_x_fm)) DEALLOCATE(humrel_daily_x_fm)
  IF (ALLOCATED(litterhum_daily_fm))  DEALLOCATE(litterhum_daily_fm)
  IF (ALLOCATED(t2m_daily_fm))  DEALLOCATE(t2m_daily_fm)
  IF (ALLOCATED(t2m_min_daily_fm))  DEALLOCATE(t2m_min_daily_fm)
  IF (ALLOCATED(tsurf_daily_fm)) DEALLOCATE(tsurf_daily_fm)
  IF (ALLOCATED(tsoil_daily_fm)) DEALLOCATE(tsoil_daily_fm)
  IF (ALLOCATED(soilhum_daily_fm))  DEALLOCATE(soilhum_daily_fm)
  IF (ALLOCATED(precip_fm)) DEALLOCATE(precip_fm)
  IF (ALLOCATED(gpp_daily_x_fm))  DEALLOCATE(gpp_daily_x_fm)
  IF (ALLOCATED(resp_maint_part_x_fm))  DEALLOCATE(resp_maint_part_x_fm)
  IF (ALLOCATED(veget_x_fm)) DEALLOCATE(veget_x_fm)
  IF (ALLOCATED(veget_max_x_fm)) DEALLOCATE(veget_max_x_fm)
  IF (ALLOCATED(lai_x_fm))  DEALLOCATE(lai_x_fm)
  
  IF (is_root_prc) THEN
     IF (ALLOCATED(clay_fm_g)) DEALLOCATE(clay_fm_g)
     IF (ALLOCATED(humrel_daily_x_fm_g)) DEALLOCATE(humrel_daily_x_fm_g)
     IF (ALLOCATED(litterhum_daily_fm_g))  DEALLOCATE(litterhum_daily_fm_g)
     IF (ALLOCATED(t2m_daily_fm_g))  DEALLOCATE(t2m_daily_fm_g)
     IF (ALLOCATED(t2m_min_daily_fm_g))  DEALLOCATE(t2m_min_daily_fm_g)
     IF (ALLOCATED(tsurf_daily_fm_g)) DEALLOCATE(tsurf_daily_fm_g)
     IF (ALLOCATED(tsoil_daily_fm_g)) DEALLOCATE(tsoil_daily_fm_g)
     IF (ALLOCATED(soilhum_daily_fm_g))  DEALLOCATE(soilhum_daily_fm_g)
     IF (ALLOCATED(precip_fm_g)) DEALLOCATE(precip_fm_g)
     IF (ALLOCATED(gpp_daily_x_fm_g))  DEALLOCATE(gpp_daily_x_fm_g)
     IF (ALLOCATED(resp_maint_part_x_fm_g))  DEALLOCATE(resp_maint_part_x_fm_g)
     IF (ALLOCATED(veget_x_fm_g)) DEALLOCATE(veget_x_fm_g)
     IF (ALLOCATED(veget_max_x_fm_g)) DEALLOCATE(veget_max_x_fm_g)
     IF (ALLOCATED(lai_x_fm_g))  DEALLOCATE(lai_x_fm_g)
  ENDIF

  IF (ALLOCATED(isf)) DEALLOCATE(isf)
  IF (ALLOCATED(nf_written)) DEALLOCATE(nf_written)
  IF (ALLOCATED(nf_cumul)) DEALLOCATE(nf_cumul)
  IF (ALLOCATED(times)) DEALLOCATE(times)
  IF (ALLOCATED(nforce)) DEALLOCATE(nforce)
  IF (ALLOCATED(control_moist)) DEALLOCATE(control_moist)
  IF (ALLOCATED(control_temp)) DEALLOCATE(control_temp)
  IF (ALLOCATED(soilcarbon_input)) DEALLOCATE(soilcarbon_input)
! for deforestation variables
  IF ( ALLOCATED (veget_max_new)) DEALLOCATE (veget_max_new)
  IF ( ALLOCATED (space_nat_new)) DEALLOCATE (space_nat_new)
  IF ( ALLOCATED (horip10_index)) DEALLOCATE (horip10_index)
  IF ( ALLOCATED (horip100_index)) DEALLOCATE (horip100_index)
  IF ( ALLOCATED (horip11_index)) DEALLOCATE (horip11_index)
  IF ( ALLOCATED (horip101_index)) DEALLOCATE (horip101_index)
  IF ( ALLOCATED (prod10)) DEALLOCATE (prod10)
  IF ( ALLOCATED (prod100)) DEALLOCATE (prod100)
  IF ( ALLOCATED (flux10)) DEALLOCATE (flux10)
  IF ( ALLOCATED (flux100)) DEALLOCATE (flux100)
  IF ( ALLOCATED (convflux)) DEALLOCATE (convflux)
  IF ( ALLOCATED (cflux_prod10)) DEALLOCATE (cflux_prod10)
  IF ( ALLOCATED (cflux_prod100)) DEALLOCATE (cflux_prod100)

! 2. reset l_first
  l_first_stomate=.TRUE.
! 3. call to clear functions
  CALL get_reftemp_clear
  CALL season_clear
  CALL stomatelpj_clear
  CALL littercalc_clear
  CALL vmax_clear
!---------------------------
END SUBROUTINE stomate_clear
!
!=
!
SUBROUTINE stomate_var_init &
 &  (kjpindex, veget, veget_max, leaf_age, leaf_frac, &
 &   tlong_ref, t2m_month, dead_leaves, &
 &   veget_x, lai_x, qsintmax, deadleaf_cover, assim_param_x, &
 &   prod10, prod100, flux10, flux100, &
 &   convflux,cflux_prod10, cflux_prod100)
! deforestation variables added as arguments

!---------------------------------------------------------------------
! this subroutine outputs values of assim_param etc.
! only if ok_stomate = .TRUE.
! otherwise,the calling procedure must do it itself.
!
! interface description
! input scalar
! Domain size
  INTEGER(i_std),INTENT(in)                            :: kjpindex
! input fields
! fractional coverage: actually covered space
  REAL(r_std),DIMENSION(kjpindex,npft),INTENT(in)       :: veget
! fractional coverage: maximum covered space
  REAL(r_std),DIMENSION(kjpindex,npft),INTENT(in)       :: veget_max
! "long term" 2 meter reference temperatures (K)
  REAL(r_std),DIMENSION(kjpindex),INTENT(in)            :: tlong_ref
! "monthly" 2 meter temperatures (K)
  REAL(r_std),DIMENSION(kjpindex),INTENT(in)            :: t2m_month
! dead leaves on ground, per PFT, metabolic and structural,
!  in gC/(m**2 of nat/agri ground)
  REAL(r_std),DIMENSION(kjpindex,npft,nlitt),INTENT(in) :: dead_leaves
! Fraction of vegetation type
  REAL(r_std),DIMENSION(kjpindex,nvm),INTENT(in)        :: veget_x
! Surface foliere
  REAL(r_std),DIMENSION(kjpindex,nvm),INTENT(in)        :: lai_x
! modified fields (actually NOT modified)
! leaf age (d)
  REAL(r_std),DIMENSION(kjpindex,npft,nleafages),INTENT(inout) :: &
 &  leaf_age
! fraction of leaves in leaf age class
  REAL(r_std),DIMENSION(kjpindex,npft,nleafages),INTENT(inout) :: &
 &  leaf_frac
! output scalar
! output fields
! Maximum water on vegetation for interception
  REAL(r_std),DIMENSION(kjpindex,nvm),INTENT(out)       :: qsintmax
! fraction of soil covered by dead leaves
  REAL(r_std),DIMENSION(kjpindex), INTENT (out)         :: deadleaf_cover
! min+max+opt temps & vmax for photosynthesis
  REAL(r_std),DIMENSION(kjpindex,nvm,npco2),INTENT(out) :: assim_param_x

! deforestation variables
! products remaining in the 10/100 year-turnover pool after the annual release for each compartment
! (10 or 100 + 1 : input from year of deforestation)
  REAL(r_std),DIMENSION(kjpindex,0:10), INTENT (out)               :: prod10
  REAL(r_std),DIMENSION(kjpindex,0:100), INTENT (out)               :: prod100
! annual release from the 10/100 year-turnover pool compartments
  REAL(r_std),DIMENSION(kjpindex,10), INTENT (out)               :: flux10
  REAL(r_std),DIMENSION(kjpindex,100), INTENT (out)               :: flux100
! release during first year following deforestation
  REAL(r_std),DIMENSION(kjpindex), INTENT (out)               :: convflux
! total annual release from the 10/100 year-turnover pool
  REAL(r_std),DIMENSION(kjpindex), INTENT (out)               :: cflux_prod10, cflux_prod100
!-
! local declaration
!-
! dummy time step, must be zero
  REAL(r_std),PARAMETER                        :: dt_0 = 0.
  REAL(r_std),DIMENSION(kjpindex,nvm)          :: vcmax
  REAL(r_std),DIMENSION(kjpindex,nvm)          :: vjmax
! Min temperature for photosynthesis (deg C)
  REAL(r_std),DIMENSION(kjpindex,npft)         :: t_photo_min
! Opt temperature for photosynthesis (deg C)
  REAL(r_std),DIMENSION(kjpindex,npft)         :: t_photo_opt
! Max temperature for photosynthesis (deg C)
  REAL(r_std),DIMENSION(kjpindex,npft)         :: t_photo_max
! Fraction of space that is natural
  REAL(r_std),DIMENSION(kjpindex)              :: space_nat
! Index
  INTEGER(i_std)                              :: j
!---------------------------------------------------------------------

! initialisation of deforestation variables
  prod10(:,:) = 0.
  prod100(:,:) = 0.
  flux10(:,:) = 0.
  flux100(:,:) = 0.
  convflux(:) = 0.
  cflux_prod10(:) = 0.
  cflux_prod100(:) = 0.

  IF (control%ok_stomate) THEN
  !
  ! 1 photosynthesis parameters
  !
  !
  ! 1.1 vcmax
  !     only if STOMATE is activated
  !
    CALL vmax (kjpindex, dt_0, leaf_age, leaf_frac, vcmax, vjmax)
  !
  ! 1.2 assimilation temperatures
  !
    CALL assim_temp(kjpindex, tlong_ref, t2m_month, &
                    t_photo_min, t_photo_opt, t_photo_max)
  !
  ! 1.3 transform into nvm vegetation types
  !
    CALL stomate_var_xout &
 &    (kjpindex,vcmax,veget,zero,assim_param_x(:,:,ivcmax))
    CALL stomate_var_xout &
 &    (kjpindex,vjmax,veget,zero,assim_param_x(:,:,ivjmax))
    CALL stomate_var_xout &
 &    (kjpindex,t_photo_min,veget,zero,assim_param_x(:,:,itmin))
    CALL stomate_var_xout &
 &    (kjpindex,t_photo_opt,veget,zero,assim_param_x(:,:,itopt))
    CALL stomate_var_xout &
 &    (kjpindex,t_photo_max,veget,zero,assim_param_x(:,:,itmax))
  !
  ! 2 dead leaf cover
  !
  ! first recalculate fraction of natural space
    space_nat(:) = 1.0
    DO j = 1, npft
      IF (.NOT.natural(j)) THEN
        space_nat(:) = space_nat(:)-veget_max(:,j)
      ENDIF
    ENDDO
    CALL deadleaf (kjpindex, space_nat, dead_leaves, deadleaf_cover)
  !
  ! 3 qsintmax
  !
    qsintmax(:,:) = qsintcst*veget_x(:,:)*lai_x(:,:)
  ENDIF ! ok_stomate = .TRUE.
!--------------------------------
  END SUBROUTINE stomate_var_init
!
!=
!
SUBROUTINE stomate_accu &
 &  (npts, n_dim2, dt_tot, dt, ldmean, field_in, field_out)
!---------------------------------------------------------------------
!
! 0 declarations
!
! 0.1 input
!
! Domain size
  INTEGER(i_std),INTENT(in)                       :: npts
! 2nd dimension (1 or npft)
  INTEGER(i_std),INTENT(in)                       :: n_dim2
! Time step of STOMATE (days)
  REAL(r_std),INTENT(in)                           :: dt_tot
! Time step in days
  REAL(r_std),INTENT(in)                           :: dt
! Calculate mean ?
  LOGICAL,INTENT(in)                              :: ldmean
! Daily field
  REAL(r_std),DIMENSION(npts,n_dim2),INTENT(in)    :: field_in
!
! 0.2 modified field
!
! Annual field
  REAL(r_std),DIMENSION(npts,n_dim2),INTENT(inout) :: field_out
!---------------------------------------------------------------------
!
! 1 accumulation
!
  field_out(:,:) = field_out(:,:)+field_in(:,:)*dt
!
! 2 mean fields
!
  IF (ldmean) THEN
    field_out(:,:) = field_out(:,:)/dt_tot
  ENDIF
!---------------------------------------------------------------------
END SUBROUTINE stomate_accu
!
!=
!
SUBROUTINE stomate_vegconvert &
 &  (kjpindex,which_way,fraction_nobio, &
 &   veget_max_x,veget_x,veget_max,veget,fvm,fv)
!---------------------------------------------------------------------
!
! 0 declarations
!
!
! 0.1 input
!
! 0.1.1 input scalar
!
! Domain size
  INTEGER(i_std),INTENT(in)                         :: kjpindex
! veget -> veget_x ('out') or veget_x -> veget ('in') ?
  CHARACTER(LEN=*),INTENT(in)                       :: which_way
! Fraction of land covered by lakes, land ice, cities, ...
  REAL(r_std),DIMENSION(kjpindex),INTENT(in)         :: fraction_nobio
!
! 0.2 modified
!
! Max vegetation fraction of hydrological module
  REAL(r_std),DIMENSION(kjpindex,nvm),INTENT(inout)  :: veget_max_x
! Vegetation fraction of hydrological module
  REAL(r_std),DIMENSION(kjpindex,nvm),INTENT(inout)  :: veget_x
! Max vegetation fraction of STOMATE
  REAL(r_std),DIMENSION(kjpindex,npft),INTENT(inout) :: veget_max
! Vegetation fraction of STOMATE
  REAL(r_std),DIMENSION(kjpindex,npft),INTENT(inout) :: veget
! factor that converts veget_max_x into veget_max
  REAL(r_std),DIMENSION(kjpindex,npft),INTENT(inout) :: fvm
! factor that converts veget_x into veget
  REAL(r_std),DIMENSION(kjpindex,npft),INTENT(inout) :: fv
!
! 0.3 local
!
  INTEGER(i_std) :: j,jv
!---------------------------------------------------------------------
  IF (which_way == 'in') THEN
  !
  ! 1 Convert veget_x into veget
  !
    DO j = 1, npft
      WHERE ((1.-fraction_nobio(:)) > 0.0)
        veget(:,j) = veget_x(:,ipft_sechiba(j)) &
 &                  *fv(:,j)/(1.-fraction_nobio(:))
        veget_max(:,j) = veget_max_x(:,ipft_sechiba(j)) &
 &                      *fvm(:,j)/(1.-fraction_nobio(:))
      ELSEWHERE
        veget(:,j) = 0.
        veget_max(:,j) = 0.
      ENDWHERE
    ENDDO
  ELSEIF (which_way == 'out') THEN
  !
  ! 2 Convert veget into veget_x
  !    and calculate fv and fvm for next time step
  !
  ! 2.1 vegetation
  !
    veget_x(:,:) = 0.0
    veget_max_x(:,:) = 0.0
    DO j = 1, npft
      veget_x(:,ipft_sechiba(j)) = &
 &      veget_x(:,ipft_sechiba(j)) &
 &     +veget(:,j)*(1.-fraction_nobio(:))
      veget_max_x(:,ipft_sechiba(j)) = &
 &      veget_max_x(:,ipft_sechiba(j)) &
 &     +veget_max(:,j)*(1.-fraction_nobio(:))
    ENDDO
  !
  ! 2.2 bare soil
  !     for bare soil, veget_max is actually the potential minimum, as
  !      it is calculated using the potential maximum vegetation cover.
  !     veget_max has no real physical meaning anyway.
  !
    veget_max_x(:,ibare_sechiba) = 1.-fraction_nobio(:)
    veget_x(:,ibare_sechiba) = 1.-fraction_nobio(:)
    DO jv = 1, nvm
      IF (jv /= ibare_sechiba) THEN
        veget_max_x(:,ibare_sechiba) = veget_max_x(:,ibare_sechiba) &
 &                                    -veget_max_x(:,jv)
        veget_x(:,ibare_sechiba) = veget_x(:,ibare_sechiba) &
 &                                -veget_x(:,jv)
      ENDIF
    ENDDO
  ! potential minimum bare soil cover cannot be less than 0
    veget_max_x(:,ibare_sechiba) = &
 &     MAX(veget_max_x(:,ibare_sechiba),0._r_std)
  !
  ! 2.3 calculate fv and fvm
  !
    DO j=1,npft
      WHERE (veget_max_x(:,ipft_sechiba(j)) > 0.)
        fvm(:,j) = veget_max(:,j)/veget_max_x(:,ipft_sechiba(j)) &
 &                *(1.-fraction_nobio(:))
      ELSEWHERE
        fvm(:,j) = 0.0
      ENDWHERE
      WHERE (veget_x(:,ipft_sechiba(j)) > 0.)
        fv(:,j) = veget(:,j)/veget_x(:,ipft_sechiba(j)) &
 &               *(1.-fraction_nobio(:))
      ELSEWHERE
        fv(:,j) = 0.0
      ENDWHERE
    ENDDO
  ELSE
    WRITE(numout,*) 'stomate_vegconvert: which_way = ', which_way
      STOP 'Cannot go this way.'
  ENDIF
!--------------------------------
END SUBROUTINE stomate_vegconvert
!
!=
!
  SUBROUTINE stomate_var_xout (kjpindex,var,vfrac,bare_val,var_x)
!---------------------------------------------------------------------
! this subroutine outputs a variable (e.g. lai_x) on nvm vegetation
!   types given an input (e.g. lai) on npft vegetation types.
! Ponderation is done using vfrac (= veget or veget_max)
!
! 0 declarations
!
! 0.1 input
!
! 0.1.1 input scalar
!
! Domain size
  INTEGER(i_std),INTENT(in)                      :: kjpindex
! variable defined on npft
  REAL(r_std),DIMENSION(kjpindex,npft),INTENT(in) :: var
! vegetation fraction
  REAL(r_std),DIMENSION(kjpindex,npft),INTENT(in) :: vfrac
! value for bare ground
  REAL(r_std),INTENT(in)                          :: bare_val
!
! 0.2 output
! variable defined on nvm
  REAL(r_std),DIMENSION(kjpindex,nvm),INTENT(out) :: var_x
!
! 0.3 local
!
  INTEGER(i_std)                                 :: j
  REAL(r_std),DIMENSION(kjpindex,nvm)             :: vfracsum
!---------------------------------------------------------------------
!
! 1 Calculate ponderated value for vegetation types
!
  var_x(:,:) = 0.0
  vfracsum(:,:) = 0.0
  DO j = 1, npft
    var_x(:,ipft_sechiba(j)) = var_x(:,ipft_sechiba(j)) &
 &                            +var(:,j)*vfrac(:,j)
    vfracsum(:,ipft_sechiba(j)) = vfracsum(:,ipft_sechiba(j)) &
 &                               +vfrac(:,j)
  ENDDO
  WHERE (vfracsum(:,:) > 0.0)
    var_x(:,:) = var_x(:,:)/vfracsum(:,:)
  ENDWHERE
!
! 2 impose a value for bare soil
!
  var_x(:,ibare_sechiba) = bare_val
!------------------------------
END SUBROUTINE stomate_var_xout
!
!=
!
  SUBROUTINE init_forcing (kjpindex,nsfm,nsft)
!---------------------------------------------------------------------
  INTEGER(i_std),INTENT(in) :: kjpindex
  INTEGER(i_std),INTENT(in) :: nsfm
  INTEGER(i_std),INTENT(in) :: nsft
!
  LOGICAL        :: l_error
  INTEGER(i_std) :: ier
!---------------------------------------------------------------------
  l_error = .FALSE.
!
  ALLOCATE(clay_fm(kjpindex,nsfm),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(humrel_daily_x_fm(kjpindex,nvm,nsfm),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(litterhum_daily_fm(kjpindex,nsfm),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(t2m_daily_fm(kjpindex,nsfm),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(t2m_min_daily_fm(kjpindex,nsfm),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(tsurf_daily_fm(kjpindex,nsfm),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(tsoil_daily_fm(kjpindex,nbdl,nsfm),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(soilhum_daily_fm(kjpindex,nbdl,nsfm),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(precip_fm(kjpindex,nsfm),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(gpp_daily_x_fm(kjpindex,nvm,nsfm),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(resp_maint_part_x_fm(kjpindex,nvm,nparts,nsfm),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(veget_x_fm(kjpindex,nvm,nsfm),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(veget_max_x_fm(kjpindex,nvm,nsfm),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(lai_x_fm(kjpindex,nvm,nsfm),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(isf(nsfm),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(nf_written(nsft),stat=ier)
  l_error = l_error .OR. (ier /= 0)
  ALLOCATE(nf_cumul(nsft),stat=ier)
  l_error = l_error .OR. (ier /= 0)

  IF (is_root_prc) THEN
     ALLOCATE(clay_fm_g(nbp_glo,nsfm),stat=ier)
     l_error = l_error .OR. (ier /= 0)
     ALLOCATE(humrel_daily_x_fm_g(nbp_glo,nvm,nsfm),stat=ier)
     l_error = l_error .OR. (ier /= 0)
     ALLOCATE(litterhum_daily_fm_g(nbp_glo,nsfm),stat=ier)
     l_error = l_error .OR. (ier /= 0)
     ALLOCATE(t2m_daily_fm_g(nbp_glo,nsfm),stat=ier)
     l_error = l_error .OR. (ier /= 0)
     ALLOCATE(t2m_min_daily_fm_g(nbp_glo,nsfm),stat=ier)
     l_error = l_error .OR. (ier /= 0)
     ALLOCATE(tsurf_daily_fm_g(nbp_glo,nsfm),stat=ier)
     l_error = l_error .OR. (ier /= 0)
     ALLOCATE(tsoil_daily_fm_g(nbp_glo,nbdl,nsfm),stat=ier)
     l_error = l_error .OR. (ier /= 0)
     ALLOCATE(soilhum_daily_fm_g(nbp_glo,nbdl,nsfm),stat=ier)
     l_error = l_error .OR. (ier /= 0)
     ALLOCATE(precip_fm_g(nbp_glo,nsfm),stat=ier)
     l_error = l_error .OR. (ier /= 0)
     ALLOCATE(gpp_daily_x_fm_g(nbp_glo,nvm,nsfm),stat=ier)
     l_error = l_error .OR. (ier /= 0)
     ALLOCATE(resp_maint_part_x_fm_g(nbp_glo,nvm,nparts,nsfm),stat=ier)
     l_error = l_error .OR. (ier /= 0)
     ALLOCATE(veget_x_fm_g(nbp_glo,nvm,nsfm),stat=ier)
     l_error = l_error .OR. (ier /= 0)
     ALLOCATE(veget_max_x_fm_g(nbp_glo,nvm,nsfm),stat=ier)
     l_error = l_error .OR. (ier /= 0)
     ALLOCATE(lai_x_fm_g(nbp_glo,nvm,nsfm),stat=ier)
     l_error = l_error .OR. (ier /= 0)
  ENDIF
!
  IF (l_error) THEN
     WRITE(numout,*) 'Problem with memory allocation: forcing variables'
     STOP 'init_forcing'
  ENDIF
!
  CALL forcing_zero
!--------------------------
END SUBROUTINE init_forcing
!
!=
!
  SUBROUTINE forcing_zero
!---------------------------------------------------------------------
  clay_fm(:,:) = zero
  humrel_daily_x_fm(:,:,:) = zero
  litterhum_daily_fm(:,:) = zero
  t2m_daily_fm(:,:) = zero
  t2m_min_daily_fm(:,:) = zero
  tsurf_daily_fm(:,:) = zero
  tsoil_daily_fm(:,:,:) = zero
  soilhum_daily_fm(:,:,:) = zero
  precip_fm(:,:) = zero
  gpp_daily_x_fm(:,:,:) = zero
  resp_maint_part_x_fm(:,:,:,:)=zero
  veget_x_fm(:,:,:) = zero
  veget_max_x_fm(:,:,:) = zero
  lai_x_fm(:,:,:) = zero
!--------------------------------
  END SUBROUTINE forcing_zero
!
!=
!
  SUBROUTINE forcing_write(forcing_id,ibeg,iend)
    !---------------------------------------------------------------------
    INTEGER(i_std),INTENT(in)  :: forcing_id
    INTEGER(i_std),INTENT(in)  :: ibeg, iend
    !
    INTEGER(i_std)                 :: iisf, iblocks, nblocks
    INTEGER(i_std)                 :: ier
    INTEGER(i_std),DIMENSION(0:2)  :: ifirst, ilast
    INTEGER(i_std),PARAMETER       :: ndm = 10
    INTEGER(i_std),DIMENSION(ndm)  :: start, count_force
    INTEGER(i_std)                 :: ndim, vid
    !---------------------------------------------------------------------
    !
    ! determine blocks of forcing states that are contiguous in memory
    !
    nblocks = 0
    ifirst(:) = 1
    ilast(:) = 1
    !
    DO iisf = ibeg, iend
       IF (     (nblocks /= 0) &
            &      .AND.(isf(iisf) == isf(ilast(nblocks))+1)) THEN
          ! element is contiguous with last element found
          ilast(nblocks) = iisf
       ELSE
          ! found first element of new block
          nblocks = nblocks+1
          IF (nblocks > 2)  STOP 'Problem in forcing_write'
          ifirst(nblocks) = iisf
          ilast(nblocks) = iisf
       ENDIF
    ENDDO
    !
    CALL gather(clay_fm,clay_fm_g)
    CALL gather(humrel_daily_x_fm,humrel_daily_x_fm_g)
    CALL gather(litterhum_daily_fm,litterhum_daily_fm_g)
    CALL gather(t2m_daily_fm,t2m_daily_fm_g)
    CALL gather(t2m_min_daily_fm,t2m_min_daily_fm_g)
    CALL gather(tsurf_daily_fm,tsurf_daily_fm_g)
    CALL gather(tsoil_daily_fm,tsoil_daily_fm_g)
    CALL gather(soilhum_daily_fm,soilhum_daily_fm_g)
    CALL gather(precip_fm,precip_fm_g)
    CALL gather(gpp_daily_x_fm,gpp_daily_x_fm_g)
    CALL gather(resp_maint_part_x_fm,resp_maint_part_x_fm_g)
    CALL gather(veget_x_fm,veget_x_fm_g)
    CALL gather(veget_max_x_fm,veget_max_x_fm_g)
    CALL gather(lai_x_fm,lai_x_fm_g)
    IF (is_root_prc) THEN
       DO iblocks = 1, nblocks
          IF (ifirst(iblocks) /= ilast(iblocks)) THEN
             ndim = 2
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(clay_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'clay',vid)
             ier = NF90_PUT_VAR (forcing_id,vid, &
                  &              clay_fm_g(:,ifirst(iblocks):ilast(iblocks)), &
                  & start=start(1:ndim), count=count_force(1:ndim))
             ndim = 3;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(humrel_daily_x_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'humrel',vid)
             ier = NF90_PUT_VAR (forcing_id, vid, &
                  &            humrel_daily_x_fm_g(:,:,ifirst(iblocks):ilast(iblocks)), &
                  &            start=start(1:ndim), count=count_force(1:ndim))
             ndim = 2;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(litterhum_daily_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'litterhum',vid)
             ier = NF90_PUT_VAR (forcing_id, vid, &
                  &            litterhum_daily_fm_g(:,ifirst(iblocks):ilast(iblocks)), &
                  & start=start(1:ndim), count=count_force(1:ndim))
             ndim = 2;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(t2m_daily_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'t2m',vid)
             ier = NF90_PUT_VAR (forcing_id, vid, &
                  &            t2m_daily_fm_g(:,ifirst(iblocks):ilast(iblocks)), &
                  & start=start(1:ndim), count=count_force(1:ndim))
             ndim = 2;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(t2m_min_daily_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'t2m_min',vid)
             ier = NF90_PUT_VAR (forcing_id, vid, &
                  &            t2m_min_daily_fm_g(:,ifirst(iblocks):ilast(iblocks)), &
                  & start=start(1:ndim), count=count_force(1:ndim))
             ndim = 2;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(tsurf_daily_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'tsurf',vid)
             ier = NF90_PUT_VAR (forcing_id, vid, &
                  &            tsurf_daily_fm_g(:,ifirst(iblocks):ilast(iblocks)), &
                  & start=start(1:ndim), count=count_force(1:ndim))
             ndim = 3;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(tsoil_daily_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'tsoil',vid)
             ier = NF90_PUT_VAR (forcing_id, vid, &
                  &            tsoil_daily_fm_g(:,:,ifirst(iblocks):ilast(iblocks)), &
                  & start=start(1:ndim), count=count_force(1:ndim))
             ndim = 3;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(soilhum_daily_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'soilhum',vid)
             ier = NF90_PUT_VAR (forcing_id, vid, &
                  &            soilhum_daily_fm_g(:,:,ifirst(iblocks):ilast(iblocks)), &
                  & start=start(1:ndim), count=count_force(1:ndim))
             ndim = 2;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(precip_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'precip',vid)
             ier = NF90_PUT_VAR (forcing_id, vid, &
                  &            precip_fm_g(:,ifirst(iblocks):ilast(iblocks)), &
                  & start=start(1:ndim), count=count_force(1:ndim))
             ndim = 3;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(gpp_daily_x_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'gpp',vid)
             ier = NF90_PUT_VAR (forcing_id, vid, &
                  &            gpp_daily_x_fm_g(:,:,ifirst(iblocks):ilast(iblocks)), &
                  &            start=start(1:ndim), count=count_force(1:ndim))
             ndim = 4;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim)=SHAPE(resp_maint_part_x_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'resp_maint_part',vid)
             ier = NF90_PUT_VAR (forcing_id,vid, &
                  &           resp_maint_part_x_fm_g(:,:,:,ifirst(iblocks):ilast(iblocks)), &
                  &           start=start(1:ndim), count=count_force(1:ndim))
             ndim = 3;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(veget_x_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'veget',vid)
             ier = NF90_PUT_VAR (forcing_id, vid, &
                  &            veget_x_fm_g(:,:,ifirst(iblocks):ilast(iblocks)), &
                  &            start=start(1:ndim), count=count_force(1:ndim))
             ndim = 3;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(veget_max_x_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'veget_max',vid)
             ier = NF90_PUT_VAR (forcing_id, vid, &
                  &            veget_max_x_fm_g(:,:,ifirst(iblocks):ilast(iblocks)), &
                  &            start=start(1:ndim), count=count_force(1:ndim))
             ndim = 3;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(lai_x_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'lai',vid)
             ier = NF90_PUT_VAR (forcing_id, vid, &
                  &            lai_x_fm_g(:,:,ifirst(iblocks):ilast(iblocks)), &
                  &            start=start(1:ndim), count=count_force(1:ndim))
          ENDIF
       ENDDO
    ENDIF
    nf_written(isf(:)) = .TRUE.
    !---------------------------
  END SUBROUTINE forcing_write
  !
  !=
  !
  SUBROUTINE forcing_read(forcing_id,nsfm)
    !---------------------------------------------------------------------
    INTEGER(i_std),INTENT(in) :: forcing_id
    INTEGER(i_std),INTENT(in) :: nsfm
    !
    INTEGER(i_std)                :: iisf, iblocks, nblocks
    INTEGER(i_std)                :: ier
    INTEGER(i_std),DIMENSION(0:2) :: ifirst, ilast
    INTEGER(i_std),PARAMETER      :: ndm = 10
    INTEGER(i_std),DIMENSION(ndm) :: start, count_force
    INTEGER(i_std)                :: ndim, vid
    !---------------------------------------------------------------------
    !
    ! set to zero if the corresponding forcing state
    ! has not yet been written into the file
    !
    DO iisf = 1, nsfm
       IF (.NOT.nf_written(isf(iisf))) THEN
          clay_fm(:,iisf) = zero
          humrel_daily_x_fm(:,:,iisf) = zero
          litterhum_daily_fm(:,iisf) = zero
          t2m_daily_fm(:,iisf) = zero
          t2m_min_daily_fm(:,iisf) = zero
          tsurf_daily_fm(:,iisf) = zero
          tsoil_daily_fm(:,:,iisf) = zero
          soilhum_daily_fm(:,:,iisf) = zero
          precip_fm(:,iisf) = zero
          gpp_daily_x_fm(:,:,iisf) = zero
          resp_maint_part_x_fm(:,:,:,iisf) = zero
          veget_x_fm(:,:,iisf) = zero
          veget_max_x_fm(:,:,iisf) = zero
          lai_x_fm(:,:,iisf) = zero
       ENDIF
    ENDDO
    !
    ! determine blocks of forcing states that are contiguous in memory
    !
    nblocks = 0
    ifirst(:) = 1
    ilast(:) = 1
    !
    DO iisf = 1, nsfm
       IF (nf_written(isf(iisf))) THEN
          IF (     (nblocks /= 0) &
               &        .AND.(isf(iisf) == isf(ilast(nblocks))+1)) THEN
             ! element is contiguous with last element found
             ilast(nblocks) = iisf
          ELSE
             ! found first element of new block
             nblocks = nblocks+1
             IF (nblocks > 2)  STOP 'Problem in forcing_read'
             !
             ifirst(nblocks) = iisf
             ilast(nblocks) = iisf
          ENDIF
       ENDIF
    ENDDO
    !
    IF (is_root_prc) THEN
       DO iblocks = 1, nblocks
          IF (ifirst(iblocks) /= ilast(iblocks)) THEN
             ndim = 2;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(clay_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'clay',vid)
             ier = NF90_GET_VAR (forcing_id, vid, &
                  &            clay_fm_g(:,ifirst(iblocks):ilast(iblocks)), &
                  &            start=start(1:ndim), count=count_force(1:ndim))
             ndim = 3;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(humrel_daily_x_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'humrel',vid)
             ier = NF90_GET_VAR (forcing_id, vid, &
                  &              humrel_daily_x_fm_g(:,:,ifirst(iblocks):ilast(iblocks)), &
                  &              start=start(1:ndim), count=count_force(1:ndim))
             ndim = 2;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(litterhum_daily_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'litterhum',vid)
             ier = NF90_GET_VAR (forcing_id, vid, &
                  &              litterhum_daily_fm_g(:,ifirst(iblocks):ilast(iblocks)), &
                  &            start=start(1:ndim), count=count_force(1:ndim))
             ndim = 2;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(t2m_daily_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'t2m',vid)
             ier = NF90_GET_VAR (forcing_id, vid, &
                  &              t2m_daily_fm_g(:,ifirst(iblocks):ilast(iblocks)), &
                  &            start=start(1:ndim), count=count_force(1:ndim))
             ndim = 2;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(t2m_min_daily_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'t2m_min',vid)
             ier = NF90_GET_VAR (forcing_id, vid, &
                  &              t2m_min_daily_fm_g(:,ifirst(iblocks):ilast(iblocks)), &
                  &            start=start(1:ndim), count=count_force(1:ndim))
             ndim = 2;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(tsurf_daily_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'tsurf',vid)
             ier = NF90_GET_VAR (forcing_id, vid, &
                  &              tsurf_daily_fm_g(:,ifirst(iblocks):ilast(iblocks)), &
                  &            start=start(1:ndim), count=count_force(1:ndim))
             ndim = 3;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(tsoil_daily_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'tsoil',vid)
             ier = NF90_GET_VAR (forcing_id, vid, &
                  &              tsoil_daily_fm_g(:,:,ifirst(iblocks):ilast(iblocks)), &
                  &            start=start(1:ndim), count=count_force(1:ndim))
             ndim = 3;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(soilhum_daily_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'soilhum',vid)
             ier = NF90_GET_VAR (forcing_id, vid, &
                  &              soilhum_daily_fm_g(:,:,ifirst(iblocks):ilast(iblocks)), &
                  &            start=start(1:ndim), count=count_force(1:ndim))
             ndim = 2;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(precip_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'precip',vid)
             ier = NF90_GET_VAR (forcing_id, vid, &
                  &              precip_fm_g(:,ifirst(iblocks):ilast(iblocks)), &
                  &            start=start(1:ndim), count=count_force(1:ndim))
             ndim = 3;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(gpp_daily_x_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'gpp',vid)
             ier = NF90_GET_VAR (forcing_id, vid, &
                  &            gpp_daily_x_fm_g(:,:,ifirst(iblocks):ilast(iblocks)), &
                  &            start=start(1:ndim), count=count_force(1:ndim))
             ndim = 4;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim)=SHAPE(resp_maint_part_x_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'resp_maint_part',vid)
             ier = NF90_GET_VAR (forcing_id,vid, &
                  &         resp_maint_part_x_fm_g(:,:,:,ifirst(iblocks):ilast(iblocks)), &
                  &              start=start(1:ndim), count=count_force(1:ndim))
             ndim = 3;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(veget_x_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'veget',vid)
             ier = NF90_GET_VAR (forcing_id, vid, &
                  &            veget_x_fm_g(:,:,ifirst(iblocks):ilast(iblocks)), &
                  &            start=start(1:ndim), count=count_force(1:ndim))
             ndim = 3;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(veget_max_x_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'veget_max',vid)
             ier = NF90_GET_VAR (forcing_id, vid, &
                  &            veget_max_x_fm_g(:,:,ifirst(iblocks):ilast(iblocks)), &
                  &            start=start(1:ndim), count=count_force(1:ndim))
             ndim = 3;
             start(1:ndim) = 1; start(ndim) = isf(ifirst(iblocks));
             count_force(1:ndim) = SHAPE(lai_x_fm_g)
             count_force(ndim) = isf(ilast(iblocks))-isf(ifirst(iblocks))+1
             ier = NF90_INQ_VARID (forcing_id,'lai',vid)
             ier = NF90_GET_VAR (forcing_id, vid, &
                  &            lai_x_fm_g(:,:,ifirst(iblocks):ilast(iblocks)), &
                  &            start=start(1:ndim), count=count_force(1:ndim))
          ENDIF
       ENDDO
    ENDIF
    CALL scatter(clay_fm_g,clay_fm)
    CALL scatter(humrel_daily_x_fm_g,humrel_daily_x_fm)
    CALL scatter(litterhum_daily_fm_g,litterhum_daily_fm)
    CALL scatter(t2m_daily_fm_g,t2m_daily_fm)
    CALL scatter(t2m_min_daily_fm_g,t2m_min_daily_fm)
    CALL scatter(tsurf_daily_fm_g,tsurf_daily_fm)
    CALL scatter(tsoil_daily_fm_g,tsoil_daily_fm)
    CALL scatter(soilhum_daily_fm_g,soilhum_daily_fm)
    CALL scatter(precip_fm_g,precip_fm)
    CALL scatter(gpp_daily_x_fm_g,gpp_daily_x_fm)
    CALL scatter(resp_maint_part_x_fm_g,resp_maint_part_x_fm)
    CALL scatter(veget_x_fm_g,veget_x_fm)
    CALL scatter(veget_max_x_fm_g,veget_max_x_fm)
    CALL scatter(lai_x_fm_g,lai_x_fm)

    !--------------------------
  END SUBROUTINE forcing_read
!
!=
!
SUBROUTINE setlai(npts,lai)
!---------------------------------------------------------------------
! routine to force the lai in STOMATE (for assimilation procedures)
! for the moment setlai only gives the lai from stomate,
! this routine must be written in the future
!
! 0 declarations
!
! 0.1 input
!
! Domain size
  INTEGER(i_std),INTENT(in)                    :: npts
! 0.3 output
  REAL(r_std),DIMENSION(npts,npft),INTENT(out)  :: lai
! 0.4 local definitions
  INTEGER(i_std)                               :: j
!---------------------------------------------------------------------
  DO j=1,npft
    WHERE (veget_max(:,j) > 0.0)
      lai(:,j) = biomass(:,j,ileaf)/veget_max(:,j)*sla(j)
    ELSEWHERE
      lai(:,j) = 0.0
    ENDWHERE
  ENDDO
!--------------------
END SUBROUTINE setlai
!
!=
!
!! BEGINNVADD
!
!=====================================================================
! NV clowproc_soil permet de faire l'interface entre sechiba
! et stomate lorsque l'on appelle uniquement le calcul du carbon sol
!
SUBROUTINE stomate_soil &
 &  (npts, dt, clay_r,space_nat_r,&
 &   soilcarbon_input, control_temp, control_moist,resp_hetero_soil)
!---------------------------------------------------------------------
!
! 0 declarations
!
! 0.1 input
!
! Domain size
  INTEGER(i_std),INTENT(in)  :: npts
! time step in days
  REAL(r_std),INTENT(in)                  :: dt
! quantity of carbon going into carbon pools from litter decomposition
!  (gC/(m**2 of nat/agri ground)/day)
  REAL(r_std),DIMENSION(npts),INTENT(in)  :: clay_r
  REAL(r_std),DIMENSION(npts),INTENT(in)  :: space_nat_r
  REAL(r_std),DIMENSION(npts,ncarb,nvegtypes),INTENT(in) :: &
 &  soilcarbon_input
! temperature control of heterotrophic respiration
  REAL(r_std),DIMENSION(npts,nlevs),INTENT(in)  :: control_temp
! moisture control of heterotrophic respiration
  REAL(r_std),DIMENSION(npts,nlevs),INTENT(in)  :: control_moist
!
! 0.3 output
!
! soil heterotrophic respiration
!  (first in gC/day/m**2 of natural/agricultural ground,
!   but output in gC/day/m**2 of total ground)
  REAL(r_std),DIMENSION(npts,nvegtypes),INTENT(out) :: resp_hetero_soil
!---------------------------------------------------------------------
  write_carbonforce = .FALSE.
  CALL soilcarbon (npts, dt, clay_r, space_nat_r, &
                   soilcarbon_input, control_temp, control_moist, &
                   carbon, resp_hetero_soil)
!--------------------------
END SUBROUTINE stomate_soil
!
!=
!
SUBROUTINE pondere_nat (npts,pondere)
!---------------------------------------------------------------------
  INTEGER(i_std),INTENT(in)                    :: npts
  REAL(r_std),DIMENSION(npts,npft),INTENT(out)  :: pondere
!
  INTEGER(i_std) :: n
!---------------------------------------------------------------------
  DO n=1,npft
    IF (natural(n)) THEN
      pondere(:,n)=space_nat(:)
    ELSE
      pondere(:,n)=1.-space_nat(:)
    ENDIF
  ENDDO
!-------------------------
END SUBROUTINE pondere_nat
!
!=
!
SUBROUTINE pondere_nat_vegfrac (npts,pondere)
!---------------------------------------------------------------------
  INTEGER(i_std),INTENT(in)                    :: npts
  REAL(r_std),DIMENSION(npts,npft),INTENT(out)  :: pondere
!
  INTEGER(i_std) :: n
!---------------------------------------------------------------------
  DO n=1,npft
    IF (natural(n)) THEN
      pondere(:,n)=space_nat(:)*veget_max(:,n)
    ELSE
      pondere(:,n)=(1.-space_nat(:))*veget_max(:,n)
    ENDIF
  ENDDO
!---------------------------------------------------------------------
  END SUBROUTINE pondere_nat_vegfrac
!
!=
!
  SUBROUTINE pondere_vegfrac (npts,pondere)
!---------------------------------------------------------------------
  INTEGER(i_std),INTENT(in)                    :: npts
  REAL(r_std),DIMENSION(npts,npft),INTENT(out)  :: pondere
!
  INTEGER(i_std) :: n
!---------------------------------------------------------------------
  DO n=1,npft
    pondere(:,n)=veget_max(:,n)
  ENDDO
!-----------------------------
END SUBROUTINE pondere_vegfrac
!! ENDNVADD
!
!=
!
!-----------------
END MODULE stomate
